//
//  AlertsView.h
//  Solscape
//
//  Created by John Schilling on 11/18/04.
//  Copyright 2004 John Schilling. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "Node.h"
#import "NodeScheduler.h"
#import "ActivityLog.h"

@interface AlertsView : NSView {
        NSString                    *_dataFolderPath;
        
        NSMutableArray              *_nodes;
        NSMutableArray              *_downloadNodeQueue;
        Node                        *_selectedNode;
        
        IBOutlet NSPopUpButton      *_nodeListMenu;
        IBOutlet NSTextField        *_nodeLastUpdateField;
        IBOutlet NSTextView         *_nodeDataTextView;
        IBOutlet NSTextField        *_nodeStatusField;
        
        int                         _lastAlertMenuIndex;
        
        IBOutlet NSButton           *_cancelNodeDownloadButton;
        IBOutlet NSButton           *_scheduleNodeButton;
        
        IBOutlet NodeScheduler      *_nodeSchedulerWindow;
        
        IBOutlet ActivityLog        *_log;
        
}

- (void)loadFrameSize;
- (void)saveFrameSize;
- (void)loadDictionaryData:(NSString *)dictFileName dataFolderPath:(NSString *)dataFolderPath;
- (void)cleanup;

- (void)addNewNodeWithDict:(NSDictionary *)nodeDict;
- (void)destroyAllNodes;
- (void)createMenuItemsForNodes;


// User Interface Methods
- (void)updateInterfaceForSelection;
- (IBAction)selectNodeFromList:(id)sender;
- (IBAction)refreshAllNodeData:(id)sender;
- (IBAction)refreshSelectedNodeData:(id)sender;
- (IBAction)refreshDailyWeatherNode:(id)sender;
- (IBAction)cancelSelectedNodeDownload:(id)sender;
- (IBAction)cancelAllDownloads:(id)sender;
- (IBAction)emptyGroupCache:(id)sender;

// TextView methods
- (void)setTextForSelectedNode:(Node *)node;
- (void)setTextViewText:(NSString *)text;
- (void)clearTextViewText;

// Text Parsing Methods
- (NSString *)munchDailySpaceWeatherReportFromString:(NSString *)input;
- (NSString *)munchSolarActivityHighlightsFromString:(NSString *)input;
- (NSString *)munchOtherReportsFromString:(NSString *)input;

// Scheduling
- (IBAction)openScheduleWindowForNode:(id)sender;
- (void)schedulerDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo;
- (IBAction)runScheduledUpdates:(id)sender;
- (IBAction)runStartupDownloads:(id)sender;

// Download Queue
- (void)addNodeToDownloadQueue:(Node *)aNode;
- (BOOL)nodeIsInDownloadQueue:(Node *)aNode;
- (void)removeNodeFromDownloadQueue:(Node *)aNode;
- (void)checkDownloadQueue;

// Node delegate methods
- (void)nodeFileDidLoad:(Node *)node;
- (void)nodeDidFailWithError:(Node *)node error:(NSString *)error;
- (void)nodeHasMessage:(Node *)node;
- (void)nodeDidChangeUpdateSettings:(Node *)node;

@end
