//
//  AlertsView.m
//  Solscape
//
//  Created by John Schilling on 11/18/04.
//  Copyright 2004 John Schilling. All rights reserved.
//

#import "AlertsView.h"
#import "AppController.h"

@implementation AlertsView

- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        _nodes              = [[NSMutableArray alloc] init];
        _downloadNodeQueue  = [[NSMutableArray alloc] init];
        _selectedNode       = nil;
    }
    return self;
}

- (void)drawRect:(NSRect)rect {
    // Drawing code here.
}

- (void)awakeFromNib
{
    [self loadFrameSize];
}

- (void)loadFrameSize
{
    NSString *lastSizeString = [[NSUserDefaults standardUserDefaults] objectForKey:@"AlertsViewSize"];
    if (lastSizeString && [lastSizeString length] != 0) {
        NSSize sSize = NSSizeFromString(lastSizeString);
        [self setFrameSize:sSize];
    }
    _lastAlertMenuIndex = [[NSUserDefaults standardUserDefaults] integerForKey:@"lastAlertMenuIndex"];
}

- (void)saveFrameSize
{
    NSString *sizeString = NSStringFromSize([self frame].size);
    [[NSUserDefaults standardUserDefaults] setObject:sizeString forKey:@"AlertsViewSize"];
    [[NSUserDefaults standardUserDefaults] setInteger:_lastAlertMenuIndex forKey:@"lastAlertMenuIndex"];
}


- (void)loadDictionaryData:(NSString *)dictFileName dataFolderPath:(NSString *)dataFolderPath;
{
    int i;
    _dataFolderPath = [dataFolderPath retain];
    
    /* Load dictionary items */
    NSString *path = [[NSBundle mainBundle] resourcePath];
    NSDictionary *rootDict = [NSDictionary dictionaryWithContentsOfFile:[NSString stringWithFormat:@"%@/%@", path, dictFileName]];
    if (!rootDict) {
        NSLog(@"Failed to load rootDict!");
        return;
    }
    int rLen = [rootDict count];
    for (i = 0; i < rLen; i++) {
        NSString *nodeKey = [NSString stringWithFormat:@"Node%d", i];
        id nodeDict = [rootDict objectForKey:nodeKey];
        if (nodeDict) {
            [self addNewNodeWithDict:nodeDict];
        }
    }
    
    [self createMenuItemsForNodes];
    _selectedNode = [_nodes objectAtIndex:_lastAlertMenuIndex];
    [_nodeListMenu selectItemAtIndex:_lastAlertMenuIndex];
    [self setTextForSelectedNode:_selectedNode];
    [self updateInterfaceForSelection];
}



- (void)cleanup
{
    [self destroyAllNodes];
    [self saveFrameSize];
}

- (void)dealloc
{
    [_dataFolderPath release];
    _dataFolderPath = nil;
    
    _downloadNodeQueue = nil;
    _nodes = nil;
    [super dealloc];
}







#pragma mark
#pragma mark NODE DATA METHODS
#pragma mark

- (void)addNewNodeWithDict:(NSDictionary *)nodeDict
{
    Node *newNode = [[Node allocWithZone:[self zone]] initWithDictionary:nodeDict 
                                                        delegate:self
                                                        dataFolderPath:_dataFolderPath
                                                        overrideDataMismatch:NO];
    if (!newNode) {
        NSLog(@"Cannot allocate resources for new Node!");
        return;
    }
        
    [_nodes addObject:newNode];
    [newNode release];

    return;
}


- (void)destroyAllNodes
{
    if ([_nodes count] < 1) return;
    
    [_downloadNodeQueue removeAllObjects];
    [_downloadNodeQueue release];
    
    NSEnumerator 	*nodesEnum = [_nodes objectEnumerator];
    id 			aNode;

    while (aNode = [nodesEnum nextObject]) {
        if (aNode) [aNode cleanup];
    }
    
    [_nodes removeAllObjects];
    [_nodes release];
}

- (void)createMenuItemsForNodes
{
    int i;
    
    [_nodeListMenu removeAllItems];
    
    for (i = 0; i < [_nodes count]; i++) {
        Node *aNode = [_nodes objectAtIndex:i];
        if (aNode) {
            [_nodeListMenu insertItemWithTitle:[aNode nodeTitle] atIndex:i];
        }
    }
}











#pragma mark
#pragma mark INTERFACE METHODS
#pragma mark

- (void)updateInterfaceForSelection
{
    if (_selectedNode) {
        [_nodeStatusField setStringValue:[_selectedNode longStatus]];
        [_scheduleNodeButton setEnabled:YES];
        [_cancelNodeDownloadButton setEnabled:[self nodeIsInDownloadQueue:_selectedNode]];
        if ([_selectedNode autoUpdates]) {
            [_nodeLastUpdateField setStringValue:[NSString stringWithFormat:@"Last Updated: %@, Next Update: %@", [_selectedNode lastUpdate], [_selectedNode nextUpdate]]];
        } else {
            [_nodeLastUpdateField setStringValue:[NSString stringWithFormat:@"Last Updated: %@", [_selectedNode lastUpdate]]];
        }
    } else {
        [_nodeStatusField setStringValue:@""];
        [_scheduleNodeButton setEnabled:NO];
        [_cancelNodeDownloadButton setEnabled:NO];
        [_nodeLastUpdateField setStringValue:@""];
    }
}

- (IBAction)selectNodeFromList:(id)sender
{
    int index = [sender indexOfSelectedItem];
    _selectedNode = [_nodes objectAtIndex:index];
    [self setTextForSelectedNode:_selectedNode];
    [self updateInterfaceForSelection];
    _lastAlertMenuIndex = index;
}

- (IBAction)refreshAllNodeData:(id)sender
{
    if ([_nodes count] < 1) return;
    int i;
    for (i = 0; i < [_nodes count]; i++) {
        [self addNodeToDownloadQueue:[_nodes objectAtIndex:i]];
    }
}

- (IBAction)refreshSelectedNodeData:(id)sender
{
    if (_selectedNode) {
        [self addNodeToDownloadQueue:_selectedNode];
    }
}

- (IBAction)refreshDailyWeatherNode:(id)sender
{
    [self addNodeToDownloadQueue:[_nodes objectAtIndex:0]];
}

- (IBAction)cancelSelectedNodeDownload:(id)sender
{
    if (_selectedNode && [self nodeIsInDownloadQueue:_selectedNode]) {
        [_selectedNode cancelDownloading];
        [self removeNodeFromDownloadQueue:_selectedNode];
    }
    [self checkDownloadQueue]; // Any left in waiting?
}

- (IBAction)cancelAllDownloads:(id)sender
{
    if ([_nodes count] < 1) return;
    int i;
    for (i = 0; i < [_nodes count]; i++) {
        Node *aNode = [_nodes objectAtIndex:i];
        if ([self nodeIsInDownloadQueue:aNode]) {
            [aNode cancelDownloading];
            [self removeNodeFromDownloadQueue:aNode];
        }
    }
}


- (IBAction)emptyGroupCache:(id)sender
{
    if ([_nodes count] < 1) return;
    int i;
    for (i = 0; i < [_nodes count]; i++) {
        Node *aNode = [_nodes objectAtIndex:i];
        if ([self nodeIsInDownloadQueue:aNode]) {
            [aNode cancelDownloading];
            [self removeNodeFromDownloadQueue:aNode];
        }
        [aNode removeDataFile];
    }
}

















#pragma mark
#pragma mark TEXT VIEW METHODS
#pragma mark

- (void)setTextForSelectedNode:(Node *)node
{
    //stringWithContentsOfFile
    if ([node fileDownloaded]) {
        NSString *data = [NSString stringWithContentsOfFile:[node filePath]];
        if (data && [data length] > 0) {
            if ([[node fileName] isEqualToString:@"RSGA.txt"]) {
                [self setTextViewText:[self munchDailySpaceWeatherReportFromString:data]];
            } else if ([[node fileName] isEqualToString:@"WKHF.txt"]) {
                [self setTextViewText:[self munchSolarActivityHighlightsFromString:data]];
            } else {
                [self setTextViewText:[self munchOtherReportsFromString:data]];
            }
        } else {
            [self setTextViewText:@"Data not loaded. Press \"Reload\" to download current data."];
        }
    } else {
        if ([self nodeIsInDownloadQueue:node]) {
            [self setTextViewText:@"Downloading data, please wait..."];
        } else {
            if ([SharedController downloadMissingData] && [SharedController isOnline]) {
                [self addNodeToDownloadQueue:node];
                [self setTextViewText:@"Downloading data, please wait..."];
            } else {
                [self setTextViewText:@"Data not loaded. Press \"Reload\" to download current data."];
            }
        }
    }
    [self updateInterfaceForSelection];
}

- (void)setTextViewText:(NSString *)text
{    
    [self clearTextViewText];
    NSMutableAttributedString *atrText = [[[NSMutableAttributedString alloc] init] autorelease];
    [atrText appendAttributedString:[[[NSAttributedString alloc] 
                initWithString:text  
                attributes:[NSDictionary dictionaryWithObjectsAndKeys:
                            [NSFont fontWithName:@"Lucida Grande" size:10.0], 
                            NSFontAttributeName, [NSColor blackColor], 
                            NSForegroundColorAttributeName, nil]] autorelease]];
    
    if( [[_nodeDataTextView textStorage] length] > 0 ) [[_nodeDataTextView textStorage] appendAttributedString:[[[NSAttributedString alloc] initWithString:@"\n"] autorelease]];
    [[_nodeDataTextView textStorage] appendAttributedString:atrText];
}

- (void)clearTextViewText
{
    [[_nodeDataTextView textStorage] setAttributedString:[[[NSAttributedString alloc] initWithString:[NSString stringWithFormat:@""]] autorelease]];
}






#pragma mark
#pragma mark TEXT PARSING METHODS
#pragma mark

- (NSString *)munchDailySpaceWeatherReportFromString:(NSString *)input
{
    NSMutableString *report = [[NSMutableString alloc] initWithString:input];
    
    [report replaceOccurrencesOfString:@"\nIA." withString:@"\n\n1A." options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\nIB." withString:@"\n\n1B." options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\nIC." withString:@"\n\n1C." options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\nID." withString:@"\n\n1D." options:nil range:NSMakeRange(0, [report length])];
    
    [report replaceOccurrencesOfString:@"\nIIA." withString:@"\n\n2A." options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\nIIB." withString:@"\n\n2B." options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\nIIC." withString:@"\n\n2C." options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\nIID." withString:@"\n\n2D." options:nil range:NSMakeRange(0, [report length])];
    
    [report replaceOccurrencesOfString:@"\nIIIA." withString:@"\n\n3A." options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\nIIIB." withString:@"\n\n3B." options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\nIIIC." withString:@"\n\n3C." options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\nIIID." withString:@"\n\n3D." options:nil range:NSMakeRange(0, [report length])];
    
    [report replaceOccurrencesOfString:@"\nIVA." withString:@"\n\n4A." options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\nIVB." withString:@"\n\n4B." options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\nIVC." withString:@"\n\n4C." options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\nIVD." withString:@"\n\n4D." options:nil range:NSMakeRange(0, [report length])];
  
    [report replaceOccurrencesOfString:@"\nVIII." withString:@"\n\n8." options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\nVII." withString:@"\n\n7." options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\nVI." withString:@"\n\n6." options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\nV." withString:@"\n\n5." options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\nIV." withString:@"\n\n4." options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\nIII." withString:@"\n\n3." options:nil range:NSMakeRange(0, [report length])]; 
    [report replaceOccurrencesOfString:@"\nII." withString:@"\n\n2." options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\nI." withString:@"\n\n1." options:nil range:NSMakeRange(0, [report length])];

    [report replaceOccurrencesOfString:@":Product: " withString:@"" options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@":Issued:" withString:@"Issued:" options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\n# " withString:@"\n" options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\n#" withString:@"\n" options:nil range:NSMakeRange(0, [report length])];
    
    [report replaceOccurrencesOfString:@"ñ" withString:@"-" options:nil range:NSMakeRange(0, [report length])];

    return [report autorelease];
}


- (NSString *)munchSolarActivityHighlightsFromString:(NSString *)input
{
    NSMutableString *report = [[NSMutableString alloc] initWithString:input];
    
    [report replaceOccurrencesOfString:@":Product: " withString:@"" options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@":Issued:" withString:@"Issued:" options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\n# " withString:@"\t" options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\n#" withString:@"\t" options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\n: " withString:@"\t" options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\n:" withString:@"\t" options:nil range:NSMakeRange(0, [report length])];
    
    [report replaceOccurrencesOfString:@"ñ" withString:@"-" options:nil range:NSMakeRange(0, [report length])];
    
    [report replaceOccurrencesOfString:@"\n\n" withString:@"\r" options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\n" withString:@" " options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\r" withString:@"\n\n" options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\t" withString:@"\n" options:nil range:NSMakeRange(0, [report length])];
    
    return [report autorelease];
}

- (NSString *)munchOtherReportsFromString:(NSString *)input
{
    NSMutableString *report = [[NSMutableString alloc] initWithString:input];
    
    [report replaceOccurrencesOfString:@":Product: " withString:@"" options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@":Issued:" withString:@"Issued:" options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\n# " withString:@"\n" options:nil range:NSMakeRange(0, [report length])];
    [report replaceOccurrencesOfString:@"\n#" withString:@"\n" options:nil range:NSMakeRange(0, [report length])];
    
    [report replaceOccurrencesOfString:@"ñ" withString:@"-" options:nil range:NSMakeRange(0, [report length])];
    
    return [report autorelease];
}

















#pragma mark
#pragma mark SCHEDULING METHODS
#pragma mark

- (IBAction)openScheduleWindowForNode:(id)sender
{
    if (_selectedNode) {
        [_nodeSchedulerWindow setMyNode:_selectedNode];
        [NSApp beginSheet: _nodeSchedulerWindow
            modalForWindow:[self window]
            modalDelegate: self
            didEndSelector: @selector(schedulerDidEnd:returnCode:contextInfo:)
            contextInfo: nil];
    }
}

- (void)schedulerDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    [sheet orderOut:self];
}

- (IBAction)runScheduledUpdates:(id)sender
{
   if ([_nodes count] < 1) return;
    int i;
    for (i = 0; i < [_nodes count]; i++) {
        Node *aNode = [_nodes objectAtIndex:i];
        if (aNode) {
            if ([aNode checkAutoUpdate]) {
                [self addNodeToDownloadQueue:aNode];
            }
        }
    }
}

- (IBAction)runStartupDownloads:(id)sender
{
    if ([_nodes count] < 1) return;
    int i;
    for (i = 0; i < [_nodes count]; i++) {
        Node *aNode = [_nodes objectAtIndex:i];
        if (aNode) {
            if ([aNode downloadsAtStartup]) {
                [self addNodeToDownloadQueue:aNode];
            }
        }
    }
}












#pragma mark
#pragma mark DOWNLOAD QUEUE METHODS
#pragma mark

/* DOWNLOAD QUEUE:
    Rather than download all items for all views at once, each view
    has a "_downloadNodeQueue", an NSMutableArray of nodes. Each time we
    want to download a node, we add it to the queue, making sure it isn't already
    in the queue. Then, when the node is finished downloading (or errors out), we remove it from
    the queue. Then we check to see if there is another item in the queue, and is so, start it downloading.
    Canceling the node download also removes it from the queue and checks to see if the next
    node in the line is waiting to download.
*/


- (void)addNodeToDownloadQueue:(Node *)aNode
{
    // Dont add if already in there...
    int i;
    for (i = 0; i < [_downloadNodeQueue count]; i++) {
        if (aNode == [_downloadNodeQueue objectAtIndex:i]) {
            return; // Match found, DONT add to queue.
        }
    }
    [_downloadNodeQueue addObject:aNode];
    [aNode setStatusMessage:@"-" long:@"Queued..."];
    [self checkDownloadQueue];
}

- (BOOL)nodeIsInDownloadQueue:(Node *)aNode
{
    if ([_downloadNodeQueue count] < 1) return NO;
    
    /* Is node in the Queue? Used for canceling downloading. */
    
    int i;
    for (i = 0; i < [_downloadNodeQueue count]; i++) {
        if (aNode == [_downloadNodeQueue objectAtIndex:i]) {
            return YES;
        }
    }
    return NO;
}

- (void)removeNodeFromDownloadQueue:(Node *)aNode
{
    if ([_downloadNodeQueue count] < 1) return;
    
    /* Remove the node using removeObjectAtIndex so that
        we "push" the remaining nodes up to the front of the line */
    
    int i;
    for (i = 0; i < [_downloadNodeQueue count]; i++) {
        if (aNode == [_downloadNodeQueue objectAtIndex:i]) {
            [_downloadNodeQueue removeObjectAtIndex:i];
            return;
        }
    }
}

- (void)checkDownloadQueue
{
    /* Any nodes in the queue waiting to be downloaded? */
    if ([_downloadNodeQueue count] < 1) return;
    if (![[_downloadNodeQueue objectAtIndex:0] isDownloading]) {
        [[_downloadNodeQueue objectAtIndex:0] beginDownloadingData];
    }
}










#pragma mark
#pragma mark NODE DELEGATE METHODS
#pragma mark

- (void)nodeFileDidLoad:(Node *)node
{
    [self removeNodeFromDownloadQueue:node];
    [self checkDownloadQueue];
    
    if (![SharedController logErrorsOnly]) {
        NSString *message = [NSString stringWithFormat:@"Download of \"%@\" Successful", [node itemName]];
        [_log addMessageToLog:message];
    }
    
    if (_selectedNode && (node == _selectedNode)) {
        [self setTextForSelectedNode:node];
    }
}

- (void)nodeDidFailWithError:(Node *)node error:(NSString *)error
{
    [self removeNodeFromDownloadQueue:node];
    [self checkDownloadQueue];    
    [_log addErrorToLog:error];
}

- (void)nodeHasMessage:(Node *)node;
{
    if (_selectedNode && (node == _selectedNode)) {
        [self updateInterfaceForSelection];
    }
}

- (void)nodeDidChangeUpdateSettings:(Node *)node
{
    if (_selectedNode && (node == _selectedNode)) {
        [self updateInterfaceForSelection];
    }
}



@end
