//
//  AppController.h
//  Solscape
//
//  Created by John Schilling on 11/10/04.
//  Copyright 2004 John Schilling. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "GroupView.h"
#import "AlertsView.h"
#import "MoonView.h"
#import "ActivityLog.h"

#define	SharedController [AppController sharedInstance]

@interface AppController : NSWindowController {

        IBOutlet GroupView              *_sunView;
        IBOutlet GroupView              *_graphView;
        IBOutlet GroupView              *_auroraView;
        IBOutlet GroupView              *_earthView;
        IBOutlet AlertsView             *_alertsView;
        IBOutlet MoonView               *_moonView;
        
        NSMutableString                 *_defaultDescription;
        
        GroupView                       *_selectedGroupView;
                
        IBOutlet NSWindow               *_preferencesPanel;
        IBOutlet NSButton               *_prefsLogErrorsOnlyButton;
        IBOutlet NSButton               *_prefsBeepOnErrorsButton;
        IBOutlet NSButton               *_prefsDownloadMissingDataButton;
        IBOutlet NSTextField            *_timeoutValueField;
        
        BOOL                            _isOnline;
        BOOL                            _logErrorsOnly;
        BOOL                            _beepOnErrors;
        BOOL                            _downloadMissingData;
        float                           _timeoutSeconds;
        
        IBOutlet NSWindow               *_aboutBoxWindow;
        IBOutlet NSTextField            *_aboutBoxVersionNumberField;
        
        NSTimer                         *_scheduler;
        
        int                             _lastView;
        
        IBOutlet ActivityLog            *_log;
}

+ (AppController *)sharedInstance;

- (void)loadPreferences;
- (void)savePreferences;
- (IBAction)openPreferencesPanel:(id)sender;
- (IBAction)setLogErrorsOnly:(id)sender;
- (IBAction)setBeepOnErrors:(id)sender;
- (IBAction)setDownloadMissingData:(id)sender;
- (BOOL)logErrorsOnly;
- (BOOL)beepOnErrors;
- (BOOL)downloadMissingData;
- (float)timeoutSeconds;
- (BOOL)isOnline;
- (NSString *)defaultDescription;

// Interface Methods
- (IBAction)toggleOnlineStatus:(id)sender;
- (IBAction)goOnline:(id)sender;
- (IBAction)goOffline:(id)sender;
- (IBAction)setTimeoutValue:(id)sender;
- (IBAction)emptyCache:(id)sender;
- (IBAction)viewDataSourceLinkClicked:(id)sender;
- (IBAction)zoomSelectedNodeData:(id)sender;
- (IBAction)saveSelectedNodeData:(id)sender;

// Timer Methods
- (void)startScheduler;
- (void)stopScheduler;
- (void)schedulerFired:(NSTimer *)timer;
- (IBAction)runAllScheduledUpdates:(id)sender;

// Toolbar delegate methods
- (void)initToolbar;
- (IBAction)runToolbarConfig:(id)sender;
- (NSToolbarItem *)toolbar:(NSToolbar *)toolbar itemForItemIdentifier:(NSString *)itemIdentifier willBeInsertedIntoToolbar:(BOOL)flag;
- (NSArray *)toolbarDefaultItemIdentifiers:(NSToolbar*)toolbar;
- (NSArray *)toolbarAllowedItemIdentifiers:(NSToolbar*)toolbar;
- (IBAction)didSelectToolbarItem:(id)sender;

// View Setting Methods
- (void)updateAllNodes;
- (void)cancelAllDownloads;
- (void)runStartupDownloads;
- (IBAction)setGroupViewFromMenu:(id)sender;
- (void)setGroupView:(int)groupNumber;
- (void)attachView:(NSView*)view display:(BOOL)display animate:(BOOL)animate;

// About Box Stuff
- (IBAction)openAboutBox:(id)sender;
- (IBAction)openJschillingDotNetLink:(id)sender;
- (IBAction)openMailtoLink:(id)sender;
- (IBAction)openDonationLink:(id)sender;
- (IBAction)openStarryNightProLink:(id)sender;
- (IBAction)openSpaceDotComLink:(id)sender;

// Data Folder methods
- (BOOL)checkCreateFolder:(NSString *)path;
- (BOOL)folderExists:(NSString *)path;
- (NSString *)pathFromApplicationSupportFolder:(NSString *)subPath;

@end
