//
//  AppController.m
//  Solscape
//
//  Created by John Schilling on 11/10/04.
//  Copyright 2004 John Schilling. All rights reserved.
//

#import "AppController.h"

#define DEFAULT_AUTOUPDATE_CHECK_SECS   5.0

static AppController *sharedInstance = nil;

@implementation AppController

#pragma mark
#pragma mark PRIVATE INSTANCE METHODS
#pragma mark

- (id)init
{
    if (sharedInstance) {
	NSLog(@"Attempt to allocate more than one AppController object!");
	return nil;
    } else {
	[super init];
	sharedInstance          = self;
        _lastView               = -1;
    }
    return sharedInstance;
}

+ (AppController *)sharedInstance 
{
    return sharedInstance ? sharedInstance : [[self alloc] init];
}

- (void)awakeFromNib
{    
    NSString *appName       = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleExecutable"];
    NSString *versionString = [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleVersion"];
    [[self window] setTitle:[NSString stringWithFormat:@"%@ version %@", appName,versionString]];
    [_aboutBoxVersionNumberField setStringValue:[NSString stringWithFormat:@"Version Number %@", versionString]];
    
    _defaultDescription = [[NSMutableString alloc] init];
    NSString *defaultDescriptionPath = [NSString stringWithFormat:@"%@/DefaultDescription.txt", [[NSBundle mainBundle] resourcePath]];
    [_defaultDescription setString:[NSString stringWithContentsOfFile: defaultDescriptionPath]];

    NSString *dataFolderName = [NSString stringWithString:@"Solscape Data"];
    NSString *dataFolderPath = [self pathFromApplicationSupportFolder: dataFolderName];
    if (! [self checkCreateFolder:dataFolderName] ) {
        NSLog(@"Can't create Data Folder!");
    }
    
    [self loadPreferences];
    [self initToolbar];
    
    [_sunView loadDomainDataForGroup:@"Sun" groupNumber:0 dictionaryFileName:@"DataSun.plist" dataFolderPath:dataFolderPath];
    [_graphView loadDomainDataForGroup:@"Graphs" groupNumber:1 dictionaryFileName:@"DataGraphs.plist" dataFolderPath:dataFolderPath];
    [_auroraView loadDomainDataForGroup:@"Aurora" groupNumber:2 dictionaryFileName:@"DataAurora.plist" dataFolderPath:dataFolderPath];
    [_earthView loadDomainDataForGroup:@"Earth" groupNumber:3 dictionaryFileName:@"DataEarth.plist" dataFolderPath:dataFolderPath];
    [_alertsView loadDictionaryData:@"DataAlerts.plist" dataFolderPath:dataFolderPath];
}

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    [self setGroupView:_lastView];
    [self runStartupDownloads];
    [self startScheduler];
    
    /* These views need to be told when we have loaded everything needed from the NIB file */
    [_sunView applicationDidFinishLaunching];
    [_graphView applicationDidFinishLaunching];
    [_auroraView applicationDidFinishLaunching];
    [_earthView applicationDidFinishLaunching];
    
    [[self window] makeKeyAndOrderFront:self];
    [[self window] display];
}

- (void)applicationWillTerminate:(NSNotification *)aNotification
{
    [[NSURLCache sharedURLCache] removeAllCachedResponses];
    
    [_sunView cleanup];
    [_graphView cleanup];
    [_auroraView cleanup];
    [_earthView  cleanup];
    [_alertsView cleanup];
    [_moonView cleanup];
    
    [self stopScheduler];
    [_log saveWindowFrame];
    [self savePreferences];
}

- (void)applicationDidHide:(NSNotification *)aNotification
{
    /* NOP */
}

- (void)mouseDown:(NSEvent *)theEvent
{
    [[self window] makeFirstResponder:[[self window] contentView]];
}

- (BOOL)windowShouldClose:(id)sender
{
    [NSApp terminate:self];
    return YES;
}

- (BOOL)validateMenuItem:(NSMenuItem *)anItem
{
    BOOL isValid = YES;
    if ([anItem tag] == 766 || [anItem tag] == 767) {
        if (_selectedGroupView == nil || ![_selectedGroupView hasSelectedNode]) isValid = NO;
    }
    return isValid;
}

- (void)dealloc
{
    [_defaultDescription release];
    _defaultDescription = nil;
    [super dealloc];
}







#pragma mark 
#pragma mark PREFERENCES METHODS
#pragma mark 

- (void)loadPreferences
{
    
    NSDictionary *appDefaults = [NSDictionary dictionaryWithObjectsAndKeys: 
                                    @"0",   @"lastView", 
                                    @"1",   @"isOnline",
                                    @"0",   @"logErrorsOnly", 
                                    @"0",   @"beepOnErrors", 
                                    @"1",   @"downloadMissingData", 
                                    @"60",  @"timeoutSeconds", 
                                    nil];
                                    
    [[NSUserDefaults standardUserDefaults] registerDefaults:appDefaults];
    
    _lastView                       = [[NSUserDefaults standardUserDefaults] integerForKey:@"lastView"];
    _isOnline                       = [[NSUserDefaults standardUserDefaults] integerForKey:@"isOnline"];
    _logErrorsOnly                  = [[NSUserDefaults standardUserDefaults] integerForKey:@"logErrorsOnly"];
    _beepOnErrors                   = [[NSUserDefaults standardUserDefaults] integerForKey:@"beepOnErrors"];
    _downloadMissingData            = [[NSUserDefaults standardUserDefaults] integerForKey:@"downloadMissingData"];
    _timeoutSeconds                 = [[NSUserDefaults standardUserDefaults] floatForKey:@"timeoutSeconds"];    
    
    if (![[self window] setFrameUsingName:@"MainWindow"]) {
        [[self window] center];
    }
    
    [_timeoutValueField setIntValue:_timeoutSeconds];
}

- (void)savePreferences
{
    [[NSUserDefaults standardUserDefaults] setInteger:_lastView forKey:@"lastView"];
    [[NSUserDefaults standardUserDefaults] setInteger:_isOnline forKey:@"isOnline"];
    [[NSUserDefaults standardUserDefaults] setInteger:_logErrorsOnly forKey:@"logErrorsOnly"];
    [[NSUserDefaults standardUserDefaults] setInteger:_beepOnErrors forKey:@"beepOnErrors"];
    [[NSUserDefaults standardUserDefaults] setInteger:_downloadMissingData forKey:@"downloadMissingData"];
    [[NSUserDefaults standardUserDefaults] setFloat:_timeoutSeconds forKey:@"timeoutSeconds"];
    
    [[self window] saveFrameUsingName:@"MainWindow"];
    [[NSUserDefaults standardUserDefaults] synchronize];
}

- (IBAction)openPreferencesPanel:(id)sender
{
    [_prefsLogErrorsOnlyButton setState:(_logErrorsOnly ? NSOnState: NSOffState)];
    [_prefsBeepOnErrorsButton setState:(_beepOnErrors ? NSOnState: NSOffState)];
    [_prefsDownloadMissingDataButton setState:(_downloadMissingData ? NSOnState: NSOffState)];
    
    [_preferencesPanel center];
    [_preferencesPanel makeKeyAndOrderFront:self];
    [_preferencesPanel display];
}

- (IBAction)setLogErrorsOnly:(id)sender
{
    _logErrorsOnly = [sender state];
}

- (IBAction)setBeepOnErrors:(id)sender
{
    _beepOnErrors = [sender state];
}

- (IBAction)setDownloadMissingData:(id)sender
{
    _downloadMissingData = [sender state];
}

- (IBAction)setTimeoutValue:(id)sender
{
    [_timeoutValueField setIntValue:[sender intValue]];
    _timeoutSeconds = [sender intValue];
}

- (BOOL)logErrorsOnly
{
    return _logErrorsOnly;
}

- (BOOL)beepOnErrors
{
    return _beepOnErrors;
}

- (BOOL)downloadMissingData
{
    return _downloadMissingData;
}

- (float)timeoutSeconds
{
    return _timeoutSeconds;
}

- (BOOL)isOnline
{
    return _isOnline;
}

- (NSString *)defaultDescription
{
    return _defaultDescription;
}










#pragma mark 
#pragma mark GENERAL INTERFACE METHODS
#pragma mark 

- (IBAction)toggleOnlineStatus:(id)sender
{
    _isOnline = !_isOnline;
    if (_isOnline) {
        [self startScheduler];
    } else {
        [self cancelAllDownloads];
        [self stopScheduler];
    }
}

- (IBAction)goOnline:(id)sender
{
    _isOnline = YES;
    [self startScheduler];
}

- (IBAction)goOffline:(id)sender
{
    _isOnline = NO;
    [self cancelAllDownloads];
    [self stopScheduler];
}

- (IBAction)emptyCache:(id)sender
{
    NSAlert *alert = [NSAlert alertWithMessageText:@"Do you really want to empty the Solscape cache?" 
                                defaultButton:@"Empty"
                                alternateButton:@"Cancel" 
                                otherButton:nil
                                informativeTextWithFormat:@"This will require all data to be reloaded before being viewed, and all current downloads will be stopped."];
                      
    int result = [alert runModal];
    if (result == 1) {
        [_sunView emptyGroupCache:self];
        [_auroraView emptyGroupCache:self];
        [_graphView emptyGroupCache:self];
        [_earthView emptyGroupCache:self];
        [_alertsView emptyGroupCache:self];
    }
}

- (IBAction)viewDataSourceLinkClicked:(id)sender
{
    NSString *URL = [sender stringValue];
    [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:URL]];
}

- (IBAction)zoomSelectedNodeData:(id)sender
{
    if (_selectedGroupView) [_selectedGroupView zoomSelectedNodeData:self];
}


- (IBAction)saveSelectedNodeData:(id)sender
{
    if (_selectedGroupView) [_selectedGroupView saveNodeImage:self];
}








#pragma mark 
#pragma mark TIMER METHODS
#pragma mark 

- (void)startScheduler
{
    [self stopScheduler];
    _scheduler = [[NSTimer scheduledTimerWithTimeInterval:(float)DEFAULT_AUTOUPDATE_CHECK_SECS
                                target:self 
                                selector:@selector(schedulerFired:) 
                                userInfo:nil 
                                repeats:YES] retain];
}

- (void)stopScheduler
{
    if (_scheduler) [_scheduler invalidate];
    if (_scheduler) [_scheduler release];
    _scheduler = nil;
}

- (void)schedulerFired:(NSTimer *)timer
{
    [self runAllScheduledUpdates:self];
}

- (IBAction)runAllScheduledUpdates:(id)sender
{
    if (!_isOnline) return;
    [_sunView runScheduledUpdates:self];
    [_auroraView runScheduledUpdates:self];
    [_graphView runScheduledUpdates:self];
    [_earthView runScheduledUpdates:self];
    [_alertsView runScheduledUpdates:self];
}


















#pragma mark 
#pragma mark VIEW SETTINGS METHODS
#pragma mark 

- (void)updateAllNodes
{
    [_sunView refreshAllNodeData:self];
    [_graphView refreshAllNodeData:self];
    [_auroraView refreshAllNodeData:self];
    [_earthView refreshAllNodeData:self];
    [_alertsView refreshAllNodeData:self];
}

- (void)runStartupDownloads
{
    if (!_isOnline) return;
    [_sunView runStartupDownloads:self];
    [_auroraView runStartupDownloads:self];
    [_graphView runStartupDownloads:self];
    [_earthView runStartupDownloads:self];
    [_alertsView runStartupDownloads:self];
}

- (void)cancelAllDownloads
{
    [_sunView cancelAllDownloads:self];
    [_auroraView cancelAllDownloads:self];
    [_graphView cancelAllDownloads:self];
    [_earthView cancelAllDownloads:self];
    [_alertsView cancelAllDownloads:self];
}


- (IBAction)setGroupViewFromMenu:(id)sender
{
    [self setGroupView:[sender tag]];
}

- (void)setGroupView:(int)groupNumber
{
    _lastView = groupNumber;
    NSString *itemIdent;
    switch(groupNumber) {
    
        case 0: /* SUN */
            [self attachView:_sunView display:YES animate:YES];
            _selectedGroupView = _sunView;
            itemIdent = @"Sun";
        break;
        
        case 1: /* GRAPHS */
            [self attachView:_graphView display:YES animate:YES];
            _selectedGroupView = _graphView;
            itemIdent = @"Graphs";
        break;
        
        case 2: /* AURORA */
            [self attachView:_auroraView display:YES animate:YES];
            _selectedGroupView = _auroraView;
            itemIdent = @"Aurora";
        break;
        
        case 3: /* EARTH */
            [self attachView:_earthView display:YES animate:YES];
            _selectedGroupView = _earthView;
            itemIdent = @"Earth";
        break;
        
        case 4: /* Alerts */
            [self attachView:_alertsView display:YES animate:YES];
            _selectedGroupView = nil;
            itemIdent = @"Alerts";
        break;
        
        case 5: /* Moon */
            [self attachView:_moonView display:YES animate:YES];
            _selectedGroupView = nil;
            itemIdent = @"Moon";
        break;
        
        default:
            [self attachView:_sunView display:YES animate:YES];
            _selectedGroupView = _sunView;
            itemIdent = @"Sun";
        break;
    }
    
    // Set the backdrop of the current toolbar item to gray...
    #if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_3
    if( [[[self window] toolbar] respondsToSelector: @selector(setSelectedItemIdentifier:)] )
            [[[self window] toolbar] setSelectedItemIdentifier: itemIdent];
    #endif
}

- (void)attachView:(NSView*)view display:(BOOL)display animate:(BOOL)animate
{
    NSWindow* window = [self window];
    
    float toolbarSize = ([window frame].size.height - [[window contentView] frame].size.height);
    NSRect frame = [window frame];
    NSSize frameSize = [view frame].size;
    frame.origin.y += frame.size.height - frameSize.height - toolbarSize;  
    frame.size = frameSize;
    frame.size.height += toolbarSize;

    [window setContentView:view];
    //[window setFrame:frame display:display animate:animate];
    //[window display];
}













#pragma mark 
#pragma mark ABOUT BOXEN METHODS
#pragma mark

- (IBAction)openAboutBox:(id)sender
{
    [_aboutBoxWindow center];
    [_aboutBoxWindow makeKeyAndOrderFront:self];
    [_aboutBoxWindow display];
}

- (IBAction)openJschillingDotNetLink:(id)sender
{
    [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"http://www.jschilling.net"]];
    [_aboutBoxWindow close];
}

- (IBAction)openMailtoLink:(id)sender
{
    [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"mailto:john@jschilling.net?subject=Solscape"]];
    [_aboutBoxWindow close];
}

- (IBAction)openDonationLink:(id)sender
{
    [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"https://www.paypal.com/xclick/business=john%40jschilling.net&no_note=1&tax=0&currency_code=USD"]];
    [_aboutBoxWindow close];
}

- (IBAction)openStarryNightProLink:(id)sender
{
    [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"http://www.starrynight.com/"]];
    [_aboutBoxWindow close];
}

- (IBAction)openSpaceDotComLink:(id)sender
{
    [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"http://www.space.com"]];
    [_aboutBoxWindow close];
}




















#pragma mark 
#pragma mark TOOLBAR METHODS
#pragma mark 

- (void)initToolbar
{
    NSToolbar *toolbar;
    toolbar = [[NSToolbar alloc] initWithIdentifier:@"toolbar"];
    [toolbar setDelegate:self];
    
    // Make the toolbar configurable
    [toolbar setAllowsUserCustomization:YES];
    [toolbar setAutosavesConfiguration:YES];
    
    // Attach the toolbar to the window
    [[self window] setToolbar:toolbar];
    
    // toolbar is retained by the window
    [toolbar release];
}

- (IBAction)runToolbarConfig:(id)sender
{
    // Start the configuration window
    [[[self window] toolbar] runCustomizationPalette:self];
}

- (NSToolbarItem *)toolbar:(NSToolbar *)tb
    itemForItemIdentifier:(NSString *)tbit
    willBeInsertedIntoToolbar:(BOOL)flag
{
    NSToolbarItem *tbi;
    
    tbi = [[NSToolbarItem alloc]initWithItemIdentifier:tbit];
    [tbi setAction:@selector(didSelectToolbarItem:)];
    [tbi setTarget:self];
    
    if ([tbit isEqualToString:@"Sun"]) {
    
        [tbi setImage:[NSImage imageNamed:@"ToolbarSun"]];
        [tbi setLabel:@"Sun"];
        [tbi setPaletteLabel:@"Sun"];
        
    } else if ([tbit isEqualToString:@"Graphs"]) {    
    
        [tbi setImage:[NSImage imageNamed:@"ToolbarGraphs"]];
        [tbi setLabel:@"Graphs"];
        [tbi setPaletteLabel:@"Graphs"];
        
    } else if ([tbit isEqualToString:@"Aurora"]) {
        
        [tbi setImage:[NSImage imageNamed:@"ToolbarAurora"]];
        [tbi setLabel:@"Aurora"];
        [tbi setPaletteLabel:@"Aurora"];
        
    } else if ([tbit isEqualToString:@"Earth"]) {    
    
        [tbi setImage:[NSImage imageNamed:@"ToolbarEarth"]];
        [tbi setLabel:@"Earth"];
        [tbi setPaletteLabel:@"Earth"];
        
    } else if ([tbit isEqualToString:@"Moon"]) {    
    
        [tbi setImage:[NSImage imageNamed:@"ToolbarMoon"]];
        [tbi setLabel:@"Moon"];
        [tbi setPaletteLabel:@"Moon"];
        
    } else if ([tbit isEqualToString:@"Alerts"]) {
    
        [tbi setImage:[NSImage imageNamed:@"ToolbarAlerts"]];
        [tbi setLabel:@"Alerts"];
        [tbi setPaletteLabel:@"Alerts"];
        
    } else if ([tbit isEqualToString:@"Update"]) {
    
        [tbi setImage:[NSImage imageNamed:@"ToolbarUpdate"]];
        [tbi setLabel:@"Update"];
        [tbi setPaletteLabel:@"Update All Data"];
        
    } else if ([tbit isEqualToString:@"Cancel"]) {
    
        [tbi setImage:[NSImage imageNamed:@"ToolbarCancelAll"]];
        [tbi setLabel:@"Cancel"];
        [tbi setPaletteLabel:@"Cancel All Downloads"];
        
    } else if ([tbit isEqualToString:@"Online"]) {
    
        [tbi setImage:[NSImage imageNamed:@"ToolbarOnline"]];
        [tbi setLabel:@"Go Online"];
        [tbi setPaletteLabel:@"Go Online"];
        
    } else if ([tbit isEqualToString:@"Offline"]) {
    
        [tbi setImage:[NSImage imageNamed:@"ToolbarOffline"]];
        [tbi setLabel:@"Go Offline"];
        [tbi setPaletteLabel:@"Go Offline"];
        
    } else if ([tbit isEqualToString:@"Prefs"]) {
    
        [tbi setImage:[NSImage imageNamed:@"ToolbarPrefs"]];
        [tbi setLabel:@"Preferences"];
        [tbi setPaletteLabel:@"Preferences"];
        
    } else {
    
        [tbi setImage:nil];
    }

    [tbi autorelease];
    return tbi;
}

- (NSArray *)toolbarDefaultItemIdentifiers:(NSToolbar*)toolbar
{
    return [NSArray arrayWithObjects:@"Sun", @"Graphs", @"Aurora", @"Earth", @"Alerts", @"Moon", 
                                        NSToolbarFlexibleSpaceItemIdentifier, 
                                        @"Online", 
                                        @"Offline", 
                                        NSToolbarSeparatorItemIdentifier, 
                                        NSToolbarCustomizeToolbarItemIdentifier, 
                                        nil];
}

- (NSArray *)toolbarAllowedItemIdentifiers:(NSToolbar*)toolbar
{
    NSString *tbit;
        
    NSMutableArray *toolArray = [NSMutableArray array];
    
    [toolArray addObject:@"Sun"];
    [toolArray addObject:@"Graphs"];
    [toolArray addObject:@"Aurora"];
    [toolArray addObject:@"Earth"];
    [toolArray addObject:@"Alerts"];
    [toolArray addObject:@"Moon"];
    [toolArray addObject:@"Update"];
    [toolArray addObject:@"Cancel"];
    [toolArray addObject:@"Online"];
    [toolArray addObject:@"Offline"];
    [toolArray addObject:@"Prefs"];
    
    // Put in a couple of standard tool bar items
    [toolArray addObject:NSToolbarSeparatorItemIdentifier];
    [toolArray addObject:NSToolbarSpaceItemIdentifier];
    [toolArray addObject:NSToolbarFlexibleSpaceItemIdentifier];
    [toolArray addObject:NSToolbarCustomizeToolbarItemIdentifier];
    
    return toolArray;
}

// You can also disable toolbar items
- (BOOL)validateToolbarItem:(NSToolbarItem *)theItem
{
    // Here I am disabling Tool 4
    if ([[theItem itemIdentifier] isEqualToString:@"Online"]) {
        return !_isOnline;
    } else if ([[theItem itemIdentifier] isEqualToString:@"Offline"]) {
        return _isOnline;
    } else if ([[theItem itemIdentifier] isEqualToString:@"Update"]) {
        return _isOnline;
    } else {
        return YES;
    }
}

- (IBAction)didSelectToolbarItem:(id)sender
{
    if ([[sender itemIdentifier] isEqualToString:@"Sun"]) {
        [self setGroupView:0];
        _selectedGroupView = _sunView;
    } else if ([[sender itemIdentifier] isEqualToString:@"Graphs"]) {
        [self setGroupView:1];
        _selectedGroupView = _graphView;
    } else if ([[sender itemIdentifier] isEqualToString:@"Aurora"]) {
        [self setGroupView:2];
        _selectedGroupView = _auroraView;
    } else if ([[sender itemIdentifier] isEqualToString:@"Earth"]) {
        [self setGroupView:3];
        _selectedGroupView = _earthView;
    } else if ([[sender itemIdentifier] isEqualToString:@"Alerts"]) {
        [self setGroupView:4];
        _selectedGroupView = nil;
    } else if ([[sender itemIdentifier] isEqualToString:@"Moon"]) {
        [self setGroupView:5];
         _selectedGroupView = nil;
    } else if ([[sender itemIdentifier] isEqualToString:@"Update"]) {
        [self updateAllNodes];
    } else if ([[sender itemIdentifier] isEqualToString:@"Cancel"]) {
        [self cancelAllDownloads];
    } else if ([[sender itemIdentifier] isEqualToString:@"Online"]) {
        [self goOnline:nil];
    } else if ([[sender itemIdentifier] isEqualToString:@"Offline"]) {
        [self goOffline:nil];
    } else if ([[sender itemIdentifier] isEqualToString:@"Prefs"]) {
        [self openPreferencesPanel:self];
    }
}


#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_3
-(NSArray*)toolbarSelectableItemIdentifiers: (NSToolbar*)toolbar
{
    // Allows setting the backdrop of the toolbar item to gray. OSX 10.3 only.
    return [NSArray arrayWithObjects:@"Sun",@"Graphs",@"Aurora",@"Earth",@"Alerts",@"Moon", nil];
}
#endif







#pragma mark 
#pragma mark DATA FOLDER METHODS
#pragma mark 

- (BOOL)checkCreateFolder:(NSString *)path
{
    NSString *folderPath = [self pathFromApplicationSupportFolder: path];
    NSFileManager *manager = [NSFileManager defaultManager];
    
    if ( ![self folderExists:folderPath] )
    {
        return [manager createDirectoryAtPath: folderPath attributes:nil];
    } else {
        return YES;
    }
}

- (BOOL)folderExists:(NSString *)path
{
    BOOL isDir = YES;
    NSFileManager *manager = [NSFileManager defaultManager];
    return [manager fileExistsAtPath:path isDirectory:&isDir];
}

- (NSString *)pathFromApplicationSupportFolder:(NSString *)subPath
{
    NSFileManager *fileManager = [NSFileManager defaultManager];
    
    NSString *appSupportFolder = [[NSHomeDirectory() stringByAppendingPathComponent:@"Library"] stringByAppendingPathComponent:@"Application Support"];

    NSString *result = [appSupportFolder stringByAppendingPathComponent: subPath];
    return result;
}

@end
