//
//  GroupView.h
//  Solscape
//
//  Created by John Schilling on 11/12/04.
//  Copyright 2004 John Schilling. All rights reserved.
//

#import <Cocoa/Cocoa.h>
#import "Domain.h"
#import "Node.h"
#import "NodeScheduler.h"
#import "ImageController.h"
#import "ActivityLog.h"
#import "ClickableTextBlue.h"

@interface GroupView : NSView {
        NSMutableArray                  *_domains;
        NSMutableArray                  *_downloadNodeQueue;
        NSMutableArray                  *_imageWindows;
        
        NSString                        *_groupName;
        int                             _groupNumber;
        NSString                        *_dataFolderPath;
        
        Domain                          *_selectedDomain;
        Node                            *_selectedNode;
        
        NSImage                         *_noDataImage;
        NSImage                         *_downloadingImage;
        NSImage                         *_splashImage;
        
        IBOutlet NSOutlineView          *_domainsList;
        IBOutlet NSSplitView            *_splitView;
        IBOutlet NSTextView             *_nodeDescriptionTextField;
        IBOutlet NSImageView            *_imageView;
        
        IBOutlet NSTextField            *_nodeTitleField;
        IBOutlet ClickableTextBlue      *_dataSourceField;
        IBOutlet NSTextField            *_lastUpdatedField;
        
        IBOutlet NSButton               *_cancelDownloadButton;
        IBOutlet NSButton               *_refreshImageButton;
        IBOutlet NSButton               *_zoomImageButton;
        IBOutlet NSButton               *_revealImageButton;
        IBOutlet NSButton               *_scheduleNodeButton;
        
        IBOutlet NSButton               *_refreshDomainButton;
        
        IBOutlet NodeScheduler          *_nodeSchedulerWindow;
        
        IBOutlet ActivityLog            *_log;
}

- (void)applicationDidFinishLaunching;
- (void)loadFrameSize;
- (void)saveFrameSize;
- (void)cleanup;

// Instance methods
- (void)loadDomainDataForGroup:(NSString *)groupName 
                                groupNumber:(int)groupNumber
                                dictionaryFileName:(NSString *)dictFileName
                                dataFolderPath:(NSString *)dataFolderPath;
- (Domain *)createNewDomain:(NSString *)name;
- (void)addNodeToDomain:(Domain *)domain withDict:(NSDictionary *)nodeDict;
- (void)destroyAllNodes;
- (BOOL)hasSelectedNode;

// Image display methods
- (void)displayImageForNode:(Node *)node;
- (NSImage *)getScaledImageFromFileAtPath:(NSString *)filePath;

// Description TextView methods
- (void)displayDescriptionForNode:(Node *)node;
- (void)clearDescriptionField;

// User Interface methods
- (IBAction)domainListDoubleClicked:(id)sender;
- (IBAction)refreshAllNodeData:(id)sender;
- (IBAction)refreshSelectedNodeData:(id)sender;
- (IBAction)refreshSelectedDomainNodes:(id)sender;
- (IBAction)zoomSelectedNodeData:(id)sender;
- (IBAction)revealSelectedNodeData:(id)sender;
- (IBAction)saveNodeImage:(id)sender;
- (void)savePanelDidEnd:(NSSavePanel *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo;
- (IBAction)cancelSelectedNodeDownload:(id)sender;
- (IBAction)cancelAllDownloads:(id)sender;
- (IBAction)emptyGroupCache:(id)sender;
- (void)updateInterfaceForSelection;

// Download Queue
- (void)addNodeToDownloadQueue:(Node *)aNode;
- (BOOL)nodeIsInDownloadQueue:(Node *)aNode;
- (void)removeNodeFromDownloadQueue:(Node *)aNode;
- (void)checkDownloadQueue;

// Sceduling
- (IBAction)openScheduleWindowForNode:(id)sender;
- (void)schedulerDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo;
- (IBAction)runScheduledUpdates:(id)sender;
- (IBAction)runStartupDownloads:(id)sender;

// ImageWindow Methods
- (void)destroyAllImageWindows;
- (void)openImageWindowForNode:(Node *)node;
- (ImageController *)imageWindowForNode:(Node *)node;
- (void)updateImageWindowForNode:(Node *)node;

// Outlineview delegate methods
- (id)outlineView:(NSOutlineView *)outlineView 
    child:(int)index 
    ofItem:(id)item;
- (BOOL)outlineView:(NSOutlineView *)outlineView 
    isItemExpandable:(id)item;
- (int)outlineView:(NSOutlineView *)outlineView 
    numberOfChildrenOfItem:(id)item;
- (id)outlineView:(NSOutlineView *)outlineView 
    objectValueForTableColumn:(NSTableColumn *)tableColumn 
    byItem:(id)item;
- (BOOL)outlineView:(NSOutlineView *)outlineView shouldSelectItem:(id)item;
- (void)outlineViewSelectionDidChange:(NSNotification *)notification;
- (void)setLastNodeViewed:(NSString *)nodeName;
- (void)selectLastViewedNode;

// SplitView Delegate Methods
- (BOOL)splitView:(id)sender canCollapseSubview:(NSView *)subview;

@end
