//
//  GroupView.m
//  Solscape
//
//  Created by John Schilling on 11/12/04.
//  Copyright 2004 John Schilling. All rights reserved.
//

#import "GroupView.h"
#import "AppController.h"

#define ULTRA_MAX_SIZE  800



@implementation GroupView

#pragma mark
#pragma mark INSTANCE METHODS
#pragma mark


- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        _domains            = [[NSMutableArray alloc] init];
        _downloadNodeQueue  = [[NSMutableArray alloc] init];
        _imageWindows       = [[NSMutableArray alloc] init];
        _selectedNode       = nil;
        _selectedDomain     = nil;
    }
    return self;
}

- (void)awakeFromNib
{
    [_domainsList setIndentationPerLevel:9.0];
    [_domainsList setDoubleAction:@selector(domainListDoubleClicked:)];
    [_domainsList setTarget:self];
}


- (void)applicationDidFinishLaunching
{
    /* Some objects in the NIB needed for display aren't ready until we are done launching */
    [self updateInterfaceForSelection];
    [self clearDescriptionField];
}

- (void)drawRect:(NSRect)rect {
    // Drawing code here.
}

- (void)loadFrameSize
{
    NSString *lastSizeString = [[NSUserDefaults standardUserDefaults] objectForKey:[NSString stringWithFormat:@"%@ViewSize", _groupName]];
    if (lastSizeString && [lastSizeString length] != 0) {
        NSSize sSize = NSSizeFromString(lastSizeString);
        [self setFrameSize:sSize];
    }
    
    /* Load Split View Size */
    if (_splitView) {
        NSMutableArray* viewRects = [[NSUserDefaults standardUserDefaults] objectForKey:[NSString stringWithFormat:@"%@ViewSplitViewRects", _groupName]];
        NSArray* subViews = [_splitView subviews];
        int i, count;
        NSRect vFrame;
        count = MIN( [viewRects count], [subViews count]);
        for( i = 0; i < count; i++ )
        {
            vFrame = NSRectFromString( [viewRects objectAtIndex: i] );
            if( NSIsEmptyRect( vFrame ) )
            {
                vFrame = [[subViews objectAtIndex: i] frame];
                if( [_splitView isVertical] )
                    vFrame.size.width = 0;
                else
                    vFrame.size.height = 0;
            }
            [[subViews objectAtIndex: i] setFrame:vFrame];
        }
    }
}

- (void)saveFrameSize
{
    NSString *sizeString = NSStringFromSize([self frame].size);
    [[NSUserDefaults standardUserDefaults] setObject:sizeString forKey:[NSString stringWithFormat:@"%@ViewSize", _groupName]];

    /* Save Split View Size */
    if (_splitView) {
        NSMutableArray*         viewRects = [NSMutableArray array];
        NSEnumerator*           viewEnum  = [[_splitView subviews] objectEnumerator];
        NSView*                 subView;
        NSRect                  vFrame;

        while( (subView = [viewEnum nextObject]) != nil )
        {
            if( [_splitView isSubviewCollapsed: subView] )
                vFrame = NSZeroRect;
            else
                vFrame = [subView frame];
            [viewRects addObject: NSStringFromRect(vFrame)];
        }
        [[NSUserDefaults standardUserDefaults] setObject:viewRects forKey:[NSString stringWithFormat:@"%@ViewSplitViewRects", _groupName]];
    }
}

- (void)cleanup
{
    [self saveFrameSize];
    [self destroyAllNodes];
    [self destroyAllImageWindows];
}

- (void)dealloc
{
    [_groupName release];
    
    [_dataFolderPath release];
    
    [_domains removeAllObjects];
    [_domains release];
    _domains = nil;
    
    [_imageWindows removeAllObjects];
    [_imageWindows release];
    _imageWindows = nil;
    
    _downloadNodeQueue = nil;
    [super dealloc];
}









#pragma mark
#pragma mark DOMAIN DATA METHODS
#pragma mark

- (void)loadDomainDataForGroup:(NSString *)groupName 
                                groupNumber:(int)groupNumber
                                dictionaryFileName:(NSString *)dictFileName
                                dataFolderPath:(NSString *)dataFolderPath
                                
{
    int i, j, k;
        
    _groupName      = [groupName retain];
    _dataFolderPath = [dataFolderPath retain];
    _groupNumber    = groupNumber;
        
    [self loadFrameSize];
    
    // GROUP SPECIFIC ITEMS
    if (groupNumber == 0 || groupNumber == 1 ) {
        _splashImage        = [NSImage imageNamed:@"SplashSolar"];
        _noDataImage        = [NSImage imageNamed:@"DataNotLoadedSolar"];
        _downloadingImage   = [NSImage imageNamed:@"DownloadingDataSolar"];
    } else if (groupNumber == 2) {
        _splashImage        = [NSImage imageNamed:@"SplashAurora"];
        _noDataImage        = [NSImage imageNamed:@"DataNotLoadedAurora"];
        _downloadingImage   = [NSImage imageNamed:@"DownloadingDataAurora"];
    } else if (groupNumber == 3) {
        _splashImage        = [NSImage imageNamed:@"SplashEarth"];
        _noDataImage        = [NSImage imageNamed:@"DataNotLoadedEarth"];
        _downloadingImage   = [NSImage imageNamed:@"DownloadingDataEarth"];
    }
        
    [_imageView setImage:_splashImage];
    
    /* Load dictionary items */
    NSString *path = [[NSBundle mainBundle] resourcePath];
    NSDictionary *rootDict = [NSDictionary dictionaryWithContentsOfFile:[NSString stringWithFormat:@"%@/%@", path, dictFileName]];
    if (!rootDict) {
        NSLog(@"Failed to load rootDict!");
        return;
    }
    int rLen = [rootDict count];
    for (i = 0; i < rLen; i++) {
        NSString *domainKey = [NSString stringWithFormat:@"Domain%d", i];
        id domainDict = [rootDict objectForKey:domainKey];
        if (domainDict) {
            NSString *domainName = [domainDict objectForKey:@"DomainName"];
            Domain *newDomain = [self createNewDomain:domainName];
            [_domainsList expandItem:[_domains objectAtIndex:i]];
            int nLen = [domainDict count] - 1;
            for (j = 0; j < nLen; j++) {
                NSString *nodeKey = [NSString stringWithFormat:@"Node%d", j];
                id nodeDict = [domainDict objectForKey:nodeKey];
                if (nodeDict) {
                    [self addNodeToDomain:newDomain withDict:nodeDict];
                }
            }
        }
    }
    [_domainsList reloadData];
    
    // Expand all domain items
    
    for ( i = 0; i < [_domains count]; i++ ) {
        int n = [[NSUserDefaults standardUserDefaults] integerForKey:[NSString stringWithFormat:@"%@ Expanded", [[_domains objectAtIndex:i] itemName]]];
        if (n != 2) [_domainsList expandItem:[_domains objectAtIndex:i]];
    }
    
}

- (Domain *)createNewDomain:(NSString *)name
{    
    Domain *newDomain = [[Domain allocWithZone:[self zone]] init];
    [newDomain setDomainName:name];
    [_domains addObject:newDomain];
    [_domainsList reloadItem:newDomain];
    
    return [newDomain autorelease];
}

- (void)addNodeToDomain:(Domain *)domain withDict:(NSDictionary *)nodeDict
{
    Node *newNode = [[Node allocWithZone:[self zone]] initWithDictionary:nodeDict 
                                                        delegate:self
                                                        dataFolderPath:_dataFolderPath
                                                        overrideDataMismatch:NO];
    if (!newNode) {
        NSLog(@"Cannot allocate resources for new Node!");
        return;
    }
        
    [domain addNode:newNode];
    [_domainsList reloadItem:domain reloadChildren:YES];
    [newNode release];

    return;
}

- (void)destroyAllNodes
{
    if ([_domains count] < 1) return;
    
    [_downloadNodeQueue removeAllObjects];
    [_downloadNodeQueue release];

    NSEnumerator 	*domainEnum = [_domains objectEnumerator];
    id 			aDomain;

    while (aDomain = [domainEnum nextObject]) {
        int nCount = [aDomain childrenCount];
        int i;
        for (i = 0; i < nCount; i++) {
            id aNode = [aDomain childAtIndex:i];
            if (aNode) {
                [aNode cleanup];
            }
        }
    }
}

- (BOOL)hasSelectedNode
{
    return (_selectedNode != nil);
}
















#pragma mark
#pragma mark IMAGE DISPLAY METHODS
#pragma mark

- (void)displayImageForNode:(Node *)node
{
    if ([node fileDownloaded]) {
        NSImage *img = [self getScaledImageFromFileAtPath:[node filePath]];
        if (img) {
            [_imageView setImage:img];
            [self updateImageWindowForNode: node];
        } else {
            [_imageView setImage:_noDataImage];
        }
    } else {
        if ([self nodeIsInDownloadQueue:node]) {
            [_imageView setImage:_downloadingImage];
        } else {
            if ([SharedController downloadMissingData] && [SharedController isOnline]) {
                [self addNodeToDownloadQueue:node];
                [_imageView setImage:_downloadingImage];
            } else {
                [_imageView setImage:_noDataImage];
            }
        }
    }
    [self updateInterfaceForSelection];
    
}

- (NSImage *)getScaledImageFromFileAtPath:(NSString *)filePath
{
    NSImage *image = [[NSImage alloc] initWithContentsOfFile: filePath];
    if (image == nil) return nil;
    
    [image setScalesWhenResized:YES];
            
    // Make 72 dpi.
    NSImageRep *rep = [image bestRepresentationForDevice:[[NSScreen mainScreen] deviceDescription]];
    [image setSize:NSMakeSize([rep pixelsWide], [rep pixelsHigh])];
    
    NSSize imageSize = [image size];
    
    if (imageSize.width > ULTRA_MAX_SIZE || imageSize.height > ULTRA_MAX_SIZE) {
        int w , h , nw , nh;
        w = nw = imageSize.width;
        h = nh = imageSize.height;

        if (w > ULTRA_MAX_SIZE || h > ULTRA_MAX_SIZE)
        {
            float wr, hr;
            // ratios
            wr = w/(float)ULTRA_MAX_SIZE;
            hr = h/(float)ULTRA_MAX_SIZE;
            if (wr > hr) // landscape
            {
                nw = ULTRA_MAX_SIZE;
                nh = h / wr;
            }
            else // portrait
            {
                nh = ULTRA_MAX_SIZE;
                nw = w / hr;
            }
        }
        NSSize newSize = NSMakeSize(nw, nh);
        [image setSize:newSize];
    }

    return [image autorelease];
}











#pragma mark
#pragma mark DESCRIPTION TEXTVIEW METHODS
#pragma mark

- (void)displayDescriptionForNode:(Node *)node
{
    if (_nodeDescriptionTextField == nil) return;

    if (node && [node itemDescription] && [[node itemDescription] length] > 1) {
        NSAttributedString *attrString = [[NSAttributedString alloc] initWithHTML:[[node itemDescription] dataUsingEncoding:NSASCIIStringEncoding allowLossyConversion:YES] documentAttributes:(NSDictionary **) NULL];
        if (attrString) {
            [[_nodeDescriptionTextField textStorage] setAttributedString:attrString];
            [attrString release];
            attrString = nil;
        }
    } else {
        [self clearDescriptionField];
    }
}

- (void)clearDescriptionField
{
    if (_nodeDescriptionTextField == nil) return;
    
    NSAttributedString *attrString = [[NSAttributedString alloc] initWithHTML:[[SharedController defaultDescription] dataUsingEncoding:NSASCIIStringEncoding allowLossyConversion:YES] documentAttributes:(NSDictionary **) NULL];
    if (attrString) {
        [[_nodeDescriptionTextField textStorage] setAttributedString: attrString];
        [attrString release];
        attrString = nil;
    }
}














#pragma mark
#pragma mark IBOUTLET METHODS
#pragma mark

- (IBAction)domainListDoubleClicked:(id)sender
{
    int i = [_domainsList selectedRow];
    if (i == -1) return;
    
    if (_selectedNode) {
        if ([_selectedNode fileDownloaded]) {
            [self zoomSelectedNodeData:self];
        }
    } else if (_selectedDomain) {
        if ([_domainsList isItemExpanded:_selectedDomain]) {
            [_domainsList collapseItem:_selectedDomain];
        } else {
            [_domainsList expandItem:_selectedDomain];
        }
    }
}

- (IBAction)refreshAllNodeData:(id)sender
{
    if ([_domains count] < 1) return;
    
    NSEnumerator 	*domainEnum = [_domains objectEnumerator];
    id 			aDomain;

    while (aDomain = [domainEnum nextObject]) {
        int nCount = [aDomain childrenCount];
        int i;
        for (i = 0; i < nCount; i++) {
            id aNode = [aDomain childAtIndex:i];
            if (aNode) {
                [self addNodeToDownloadQueue:aNode];
            }
        }
    }
}

- (IBAction)refreshSelectedNodeData:(id)sender
{
    if (_selectedNode) {
        [self addNodeToDownloadQueue:_selectedNode];
    }
}

- (IBAction)refreshSelectedDomainNodes:(id)sender
{
    if (_selectedDomain) {
        int nCount = [_selectedDomain childrenCount];
        int i;
        for (i = 0; i < nCount; i++) {
            Node *aNode = [_selectedDomain childAtIndex:i];
            if (aNode) {
                [self addNodeToDownloadQueue:aNode];
            }
        }
    }
}


- (IBAction)zoomSelectedNodeData:(id)sender
{
    if (_selectedNode && [_selectedNode fileType] == 0) {
        [self openImageWindowForNode:_selectedNode];
    }
}

- (IBAction)revealSelectedNodeData:(id)sender
{
    if (_selectedNode && [_selectedNode fileType] == 0) {
        [[NSWorkspace sharedWorkspace] selectFile:[_selectedNode filePath] inFileViewerRootedAtPath:nil];
    }
}

- (IBAction)saveNodeImage:(id)sender
{
    if (_selectedNode && [_selectedNode fileType] == 0) {
        [[NSSavePanel savePanel]
            beginSheetForDirectory:NSHomeDirectory()  // seems like an irrational place to start..
            file:[_selectedNode fileName]
            modalForWindow:[self window]
            modalDelegate:self 
            didEndSelector:@selector(savePanelDidEnd:returnCode:contextInfo:)
            contextInfo:NULL];
    }
}

- (void)savePanelDidEnd:(NSSavePanel *)sheet returnCode:(int)returnCode contextInfo:(void  *)contextInfo
{
    if (returnCode == NSOKButton)
    {
        if (_selectedNode && [_selectedNode fileType] == 0)
        {
            NSString *sourceFile = [_selectedNode filePath];
            NSFileManager *manager = [NSFileManager defaultManager];
            if ([manager fileExistsAtPath: sourceFile])
                [manager copyPath:sourceFile toPath:[sheet filename] handler:nil];
        }
    }
}




- (IBAction)cancelSelectedNodeDownload:(id)sender
{
    if (_selectedNode && [self nodeIsInDownloadQueue:_selectedNode]) {
        [_selectedNode cancelDownloading];
        [self removeNodeFromDownloadQueue:_selectedNode];
    }
    [self checkDownloadQueue]; // Any left in waiting?
}

- (IBAction)cancelAllDownloads:(id)sender
{
    if ([_domains count] < 1) return;
    
    NSEnumerator 	*domainEnum = [_domains objectEnumerator];
    id 			aDomain;

    while (aDomain = [domainEnum nextObject]) {
        int nCount = [aDomain childrenCount];
        int i;
        for (i = 0; i < nCount; i++) {
            id aNode = [aDomain childAtIndex:i];
            if ([self nodeIsInDownloadQueue:aNode]) {
                [aNode cancelDownloading];
                [self removeNodeFromDownloadQueue:aNode];
            }
        }
    }
    // DONT check the queue, as none should be left. This is why I don't just always check in removeNodeFromDownloadQueue
}

- (IBAction)emptyGroupCache:(id)sender
{
    if ([_domains count] < 1) return;
    
    NSEnumerator 	*domainEnum = [_domains objectEnumerator];
    id 			aDomain;

    while (aDomain = [domainEnum nextObject]) {
        int nCount = [aDomain childrenCount];
        int i;
        for (i = 0; i < nCount; i++) {
            id aNode = [aDomain childAtIndex:i];
            // If downloading, stop
            if ([self nodeIsInDownloadQueue:aNode]) {
                [aNode cancelDownloading];
                [self removeNodeFromDownloadQueue:aNode];
            }
            [aNode removeDataFile];
        }
    }
}

- (void)updateInterfaceForSelection
{
    
    if (_selectedNode) {
        [_cancelDownloadButton setEnabled:[self nodeIsInDownloadQueue:_selectedNode]];
        [_refreshImageButton setEnabled:YES];
        [_zoomImageButton setEnabled:[_selectedNode fileDownloaded]];
        [_revealImageButton setEnabled:[_selectedNode fileDownloaded]];
        [_scheduleNodeButton setEnabled:YES];
        [_nodeTitleField setStringValue:[_selectedNode nodeTitle]];
        [_dataSourceField setStringValue:[_selectedNode dataSource]];
        if ([_selectedNode autoUpdates]) {
            [_lastUpdatedField setStringValue:[NSString stringWithFormat:@"Last Updated: %@, Next Update: %@", [_selectedNode lastUpdate], [_selectedNode nextUpdate]]];
        } else {
            [_lastUpdatedField setStringValue:[NSString stringWithFormat:@"Last Updated: %@", [_selectedNode lastUpdate]]];
        }
    } else {
        [_cancelDownloadButton setEnabled:NO];
        [_refreshImageButton  setEnabled:NO];
        [_zoomImageButton setEnabled:NO];
        [_revealImageButton setEnabled:NO];
        [_scheduleNodeButton setEnabled:NO];
        [_nodeTitleField setStringValue:@"Please select an item from the list."];
        [_dataSourceField setStringValue:@""];
        [_lastUpdatedField setStringValue:@""];
    }
    
    [_refreshDomainButton setEnabled:(_selectedDomain != nil)];
    
}

















#pragma mark
#pragma mark DOWNLOAD QUEUE METHODS
#pragma mark

/* DOWNLOAD QUEUE:
    Rather than download all items for all views at once, each view
    has a "_downloadNodeQueue", an NSMutableArray of nodes. Each time we
    want to download a node, we add it to the queue, making sure it isn't already
    in the queue. Then, when the node is finished downloading (or errors out), we remove it from
    the queue. Then we check to see if there is another item in the queue, and is so, start it downloading.
    Canceling the node download also removes it from the queue and checks to see if the next
    node in the line is waiting to download.
*/


- (void)addNodeToDownloadQueue:(Node *)aNode
{
    // Dont add if already in there...
    int i;
    for (i = 0; i < [_downloadNodeQueue count]; i++) {
        if (aNode == [_downloadNodeQueue objectAtIndex:i]) {
            return; // Match found, DONT add to queue.
        }
    }
    [_downloadNodeQueue addObject:aNode];
    [aNode setStatusMessage:@"-" long:@"-"];
    [self checkDownloadQueue];
}

- (BOOL)nodeIsInDownloadQueue:(Node *)aNode
{
    if ([_downloadNodeQueue count] < 1) return NO;
    
    /* Is node in the Queue? Used for canceling downloading. */
    
    int i;
    for (i = 0; i < [_downloadNodeQueue count]; i++) {
        if (aNode == [_downloadNodeQueue objectAtIndex:i]) {
            return YES;
        }
    }
    return NO;
}

- (void)removeNodeFromDownloadQueue:(Node *)aNode
{
    if ([_downloadNodeQueue count] < 1) return;
    
    /* Remove the node using removeObjectAtIndex so that
        we "push" the remaining nodes up to the front of the line */
    
    int i;
    for (i = 0; i < [_downloadNodeQueue count]; i++) {
        if (aNode == [_downloadNodeQueue objectAtIndex:i]) {
            [_downloadNodeQueue removeObjectAtIndex:i];
            return;
        }
    }
}

- (void)checkDownloadQueue
{
    /* Any nodes in the queue waiting to be downloaded? */
    if ([_downloadNodeQueue count] < 1) return;
    if (![[_downloadNodeQueue objectAtIndex:0] isDownloading]) {
        [[_downloadNodeQueue objectAtIndex:0] beginDownloadingData];
    }
}

















#pragma mark
#pragma mark SCEDULING METHODS
#pragma mark

- (IBAction)openScheduleWindowForNode:(id)sender
{
    if (_selectedNode) {
        [_nodeSchedulerWindow setMyNode:_selectedNode];
        [NSApp beginSheet: _nodeSchedulerWindow
            modalForWindow:[self window]
            modalDelegate: self
            didEndSelector: @selector(schedulerDidEnd:returnCode:contextInfo:)
            contextInfo: nil];
    }
}

- (void)schedulerDidEnd:(NSWindow *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
{
    [sheet orderOut:self];
}


- (IBAction)runScheduledUpdates:(id)sender
{
    if ([_domains count] < 1) return;
    
    NSEnumerator 	*domainEnum = [_domains objectEnumerator];
    id 			aDomain;

    while (aDomain = [domainEnum nextObject]) {
        int nCount = [aDomain childrenCount];
        int i;
        for (i = 0; i < nCount; i++) {
            id aNode = [aDomain childAtIndex:i];
            if (aNode) {
                if ([aNode checkAutoUpdate]) {
                    [self addNodeToDownloadQueue:aNode];
                }
            }
        }
    }
}

- (IBAction)runStartupDownloads:(id)sender
{
    if ([_domains count] < 1) return;
    
    NSEnumerator 	*domainEnum = [_domains objectEnumerator];
    id 			aDomain;

    while (aDomain = [domainEnum nextObject]) {
        int nCount = [aDomain childrenCount];
        int i;
        for (i = 0; i < nCount; i++) {
            id aNode = [aDomain childAtIndex:i];
            if (aNode) {
                if ([aNode downloadsAtStartup]) {
                    [self addNodeToDownloadQueue:aNode];
                }
            }
        }
    }
}







#pragma mark
#pragma mark NODE DELEGATE METHODS
#pragma mark

- (void)nodeFileDidLoad:(Node *)node
{
    
    [self removeNodeFromDownloadQueue:node];
    [self checkDownloadQueue];
    
    if (![SharedController logErrorsOnly]) {
        NSString *message = [NSString stringWithFormat:@"Download of \"%@\" Successful", [node itemName]];
        [_log addMessageToLog:message];
    }
    
    if (_selectedNode && (node == _selectedNode)) {
        [self displayImageForNode:node];
    }
}

- (void)nodeDidFailWithError:(Node *)node error:(NSString *)error
{
    [self removeNodeFromDownloadQueue:node];
    [self checkDownloadQueue];    
    [_log addErrorToLog:error];
}

- (void)nodeHasMessage:(Node *)node;
{
    [_domainsList reloadItem:node];
    if (_selectedNode && (node == _selectedNode)) {
        [self updateInterfaceForSelection];
    }
}

- (void)nodeDidChangeUpdateSettings:(Node *)node
{
    if (_selectedNode && (node == _selectedNode)) {
        [self updateInterfaceForSelection];
    }
}











#pragma mark
#pragma mark ImageWindow DATASOURCE METHODS
#pragma mark

- (void)destroyAllImageWindows
{
    if ([_imageWindows count] < 1) return;
    
    NSEnumerator 	*iwinEnum = [_imageWindows objectEnumerator];
    id 			aWin;

    while (aWin = [iwinEnum nextObject]) {
        [aWin release];
    }
}

- (void)openImageWindowForNode:(Node *)node
{
    if (![node fileDownloaded]) return;
    
    ImageController *imageWin = [self imageWindowForNode:node];
    
    if (imageWin == nil) {
        ImageController *newImageWin = [[ImageController allocWithZone:[self zone]] 
                                        initWithImagePath:[_selectedNode filePath] 
                                        name:[_selectedNode itemName]
                                        URL:[_selectedNode dataSource]
                                       node:node
                                   delegate:self];
        [_imageWindows addObject:newImageWin];
        [newImageWin release];
    } else {
        [imageWin updateWithOpen:YES];
    }
}

- (ImageController *)imageWindowForNode:(Node *)node
{
    if ([_imageWindows count] < 1) nil;
    
    NSEnumerator 	*iwinEnum = [_imageWindows objectEnumerator];
    id 			aWin;

    while (aWin = [iwinEnum nextObject]) {
        if ([aWin node] == node) {
            return aWin;
        }
    }
    return nil;
}

- (void)updateImageWindowForNode:(Node *)node
{
    if (![node fileDownloaded]) return;
    
    ImageController *imageWin = [self imageWindowForNode:node];
    
    if (imageWin != nil) [imageWin updateWithOpen:NO];
}

- (void)imageWindowDidExit:(ImageController *)ic
{
    if ([_imageWindows count] < 1) return;
    
    NSEnumerator 	*iwinEnum = [_imageWindows objectEnumerator];
    id 			aWin;

    while (aWin = [iwinEnum nextObject]) {
        if (aWin == ic) {
            [aWin release];
            break;
        }
    }
}











#pragma mark
#pragma mark OUTLINEVIEW DATASOURCE METHODS
#pragma mark

- (id)outlineView:(NSOutlineView *)ov child:(int)index ofItem:(id)item
{

    // is the parent non-nil?
    if (item) {
        // Return the child
        return [item childAtIndex:index];
    } else {
        // Else return the root
        if ([_domains objectAtIndex:index] ) {
            return [_domains objectAtIndex:index];
        } else {
            return nil;
        }
    }
}

- (BOOL)outlineView:(NSOutlineView *)ov isItemExpandable:(id)item
{
    return [item expandable];
}

- (int)outlineView:(NSOutlineView *)ov numberOfChildrenOfItem:(id)item
{
    if (item == nil) {
        // TOP LEVEL
        return [_domains count];
    }
    return [item childrenCount];
}

- (id)outlineView:(NSOutlineView *)ov objectValueForTableColumn:(NSTableColumn *)tableColumn byItem:(id)item
{
    NSString *identifier = [tableColumn identifier];
    if ([identifier isEqual:@"itemName"]) return [item itemName];
    if ([identifier isEqual:@"nodeStatus"]) return [item itemStatus];
    return nil;
}

- (BOOL)outlineView:(NSOutlineView *)ov shouldSelectItem:(id)item
{
    BOOL select = YES;

    if (item != nil) {
        if ([item isNode]) {
            _selectedDomain = nil;
            if (item != _selectedNode) {
                _selectedNode = item;
                [self setLastNodeViewed:[item itemName]];
                [self displayImageForNode:item];
                [self displayDescriptionForNode: _selectedNode];
            }
            select = YES;
        } else {
            _selectedNode = nil;
            _selectedDomain = item;
            [self setLastNodeViewed:@""];
            [_imageView setImage:_splashImage];
            [self clearDescriptionField];
        }
    } else {
        _selectedNode = nil;
        _selectedDomain = nil;
        [self setLastNodeViewed:@""];
        [_imageView setImage:_splashImage];
        [self clearDescriptionField];
    }
    
    [self updateInterfaceForSelection];
    return select;
}

- (void)outlineViewSelectionDidChange:(NSNotification *)notification
{
    int i = [_domainsList selectedRow];
    if (i == -1) {
        _selectedNode = nil;
        _selectedDomain = nil;
        [_imageView setImage:_splashImage];
        [self updateInterfaceForSelection];
        [self clearDescriptionField];
    }
}

- (BOOL)outlineView:(NSOutlineView *)outlineView shouldExpandItem:(id)item
{
    [[NSUserDefaults standardUserDefaults] setInteger:1 
        forKey:[NSString stringWithFormat:@"%@ Expanded", [item itemName]]];
    return YES;
}

- (BOOL)outlineView:(NSOutlineView *)outlineView shouldCollapseItem:(id)item
{
    [[NSUserDefaults standardUserDefaults] setInteger:2 
        forKey:[NSString stringWithFormat:@"%@ Expanded", [item itemName]]];
    return YES;
}

- (void)setLastNodeViewed:(NSString *)nodeName
{
    /* Remember last node selected for next launch */
    [[NSUserDefaults standardUserDefaults] setObject:nodeName forKey:[NSString stringWithFormat:@"%@ LastNodeViewed", _groupName]];
}

- (void)selectLastViewedNode
{
    NSString *lastNodeViewedName = [[NSUserDefaults standardUserDefaults] objectForKey:[NSString stringWithFormat:@"%@ LastNodeViewed", _groupName]];
    
    NSEnumerator 	*domainEnum = [_domains objectEnumerator];
    id 			aDomain;

    while (aDomain = [domainEnum nextObject]) {
        int nCount = [aDomain childrenCount];
        int i;
        for (i = 0; i < nCount; i++) {
            id aNode = [aDomain childAtIndex:i];
            if (aNode) {
                if ([[aNode itemName] isEqualToString:lastNodeViewedName]) {
                    //[_domainsList];
                }
            }
        }
    }
}







#pragma mark
#pragma mark SPLIVIEW DELEGATE METHODS
#pragma mark

- (BOOL)splitView:(id)sender canCollapseSubview:(NSView *)subview
{
    return (subview == [[_splitView subviews] objectAtIndex:1]);
}

@end
