//
//  ImageWindow.m
//  Solscape
//
//  Created by John Schilling on 12/9/04.
//  Copyright 2004 John Schilling. All rights reserved.
//

#import "ImageController.h"

#define RESIZE_INCREMENT        0.10
#define MIN_SIZE_MULTIPLIER     4.00
#define MAX_SIZE_MULTIPLIER     1.50
#define ULTRA_MAX_SIZE          800

@implementation ImageController

- (id)initWithImagePath:(NSString *)path 
                   name:(NSString *)name
                   URL:(NSString *)URL
                  node:(Node *)node
              delegate:(id)delegate
{

    NSImage *image = [self getScaledImageFromFileAtPath:path];
    if (!image)  {
        NSLog(@"ImageController (%@): image == nil", name);
        return nil;
    }
    
    self = [super initWithWindowNibName:@"ImageWindow"];
    if (!self) {
        NSLog(@"Cannot alloc new ImageController for %@", name);
        return nil;
    }
    
    if ([self window] == nil) {
        NSLog(@"ImageController (%@): [self window] == nil", name);
        return nil;
    } else {
        _imageWindow = (ImageWindow *)[self window];
    }
    
    _delegate       = delegate;
    _node           = node;
    _name           = [name retain];
    _path           = [path retain];
    _URL            = [URL retain];
    
    [_imageWindow setDelegate:self];
    [_imageWindow setReleasedWhenClosed: NO];
    [_imageWindow setTitle:_name];
    
    NSSize cSize = NSMakeSize([image size].width, [image size].height);
    [_imageWindow setContentSize: cSize];
    [_imageView setImage:image];
    
    [self setWindowFrameAttributes];
    
    return self;
}

- (void)setWindowFrameAttributes
{
    
    NSRect wFrame       = [_imageWindow frame];
    NSRect cFrame       = [_imageView frame];
    _dwidth             = cFrame.size.width;
    _dheight            = cFrame.size.height;
    _titleBarHeight     = wFrame.size.height - cFrame.size.height;
    
    [_imageWindow setContentMinSize:NSMakeSize((_dwidth / MIN_SIZE_MULTIPLIER), (_dheight  / MIN_SIZE_MULTIPLIER))];
    [_imageWindow setContentMaxSize:NSMakeSize((_dwidth * MAX_SIZE_MULTIPLIER), (_dheight * MAX_SIZE_MULTIPLIER))];
    [_imageWindow setContentAspectRatio:NSMakeSize(_dwidth, _dheight)];
    
    [[self window] makeKeyAndOrderFront:self];
}

- (NSImage *)getScaledImageFromFileAtPath:(NSString *)filePath
{
    NSImage *image = [[NSImage alloc] initWithContentsOfFile: filePath];
    if (image == nil) return nil;
    
    [image setScalesWhenResized:YES];
            
    // Make 72 dpi.
    NSImageRep *rep = [image bestRepresentationForDevice:[[NSScreen mainScreen] deviceDescription]];
    [image setSize:NSMakeSize([rep pixelsWide], [rep pixelsHigh])];
    
    NSSize imageSize = [image size];
    
    if (imageSize.width > ULTRA_MAX_SIZE || imageSize.height > ULTRA_MAX_SIZE) {
        int w , h , nw , nh;
        w = nw = imageSize.width;
        h = nh = imageSize.height;

        if (w > ULTRA_MAX_SIZE || h > ULTRA_MAX_SIZE)
        {
            float wr, hr;
            // ratios
            wr = w/(float)ULTRA_MAX_SIZE;
            hr = h/(float)ULTRA_MAX_SIZE;
            if (wr > hr) // landscape
            {
                nw = ULTRA_MAX_SIZE;
                nh = h / wr;
            }
            else // portrait
            {
                nh = ULTRA_MAX_SIZE;
                nw = w / hr;
            }
        }
        NSSize newSize = NSMakeSize(nw, nh);
        [image setSize:newSize];
    }

    return [image autorelease];
}

- (BOOL)windowShouldClose:(id)sender
{
    if ([_delegate respondsToSelector:@selector( imageWindowDidExit: )]) {
        //[_delegate imageWindowDidExit:self];
    }
    return YES;
}

- (void)cleanup
{    

}

- (void)dealloc
{
    [_name release];
    _name = nil;
    [_path release];
    _path = nil;
    [_URL release];
    _URL = nil;
    [super dealloc];
}








- (void)updateWithOpen:(BOOL)open
{
    NSImage *image = [self getScaledImageFromFileAtPath:_path];
    [_imageView setImage:image];
    if (open) [[self window] makeKeyAndOrderFront:self];
}








- (NSString *)path
{
    return _path;
}

- (NSString *)name
{
    return _name;
}

- (NSString *)URL
{
    return _URL;
}

- (Node *)node
{
    return _node;
}






#pragma mark
#pragma mark WINDOW SIZING METHODS
#pragma mar

- (IBAction)viewAtDefaultSize:(id)sender
{
    NSSize defaultSize = NSMakeSize(_dwidth, _dheight);
    [self zoomToNewSize:defaultSize animate:NO];
    [_imageWindow center];
}

- (IBAction)viewAtMinSize:(id)sender
{
    [self zoomToNewSize:[_imageWindow contentMinSize] animate:NO];
}

- (IBAction)viewAtMaxSize:(id)sender
{
    [self zoomToNewSize:[_imageWindow contentMaxSize] animate:NO];
}

- (IBAction)zoomIn:(id)sender
{
    NSSize cSize = [[_imageWindow contentView] frame].size;
    NSSize newSize = NSMakeSize(cSize.width / (1 + RESIZE_INCREMENT), cSize.height / (1 + RESIZE_INCREMENT));
    if ((newSize.width >= [_imageWindow contentMinSize].width) && (newSize.height >= [_imageWindow contentMinSize].height)) {
        [self zoomToNewSize:newSize animate:NO];
    } else {
        [self zoomToNewSize:[_imageWindow contentMinSize] animate:NO];
    }
}

- (IBAction)zoomOut:(id)sender
{
    NSSize cSize = [[_imageWindow contentView] frame].size;
    NSSize newSize = NSMakeSize(cSize.width * (1 + RESIZE_INCREMENT), cSize.height * (1 + RESIZE_INCREMENT));
    if ((newSize.width <= [_imageWindow contentMaxSize].width) && (newSize.height <= [_imageWindow contentMaxSize].height)) {
        [self zoomToNewSize:newSize animate:NO];
    } else {
        [self zoomToNewSize:[_imageWindow contentMaxSize] animate:NO];
    }
}

- (IBAction)miniaturizeWindow:(id)sender
{
    [_imageWindow performMiniaturize:self];
}

- (IBAction)centerWindow:(id)sender
{
    [_imageWindow center];
}

- (void)zoomToNewSize:(NSSize)newSize animate:(BOOL)animate
{
    /* 
        You know, developers shouldn't have to go through all this shit 
        just to anchor the window corner to the upper left, Apple. 
    */
        
    NSRect cRect = NSMakeRect(0, 0, newSize.width, newSize.height);
    NSRect nRect = NSMakeRect([_imageWindow frame].origin.x, 
                    [_imageWindow frame].origin.y - (cRect.size.height - [_imageWindow frame].size.height), 
                    cRect.size.width, cRect.size.height);
    [_imageWindow setFrame:nRect display:YES animate:animate];
}


- (IBAction)saveDocument:(id)sender
{

}

@end
