//
//  MoonView.m
//  Solscape
//
//  Created by John Schilling on 11/18/04.
//  Copyright 2004 John Schilling. All rights reserved.
//

#import "MoonView.h"

#define SYNODIC         29.53058867


@implementation MoonView

- (id)initWithFrame:(NSRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code here.
        _isOnToday = YES;
    }
    return self;
}

- (void)drawRect:(NSRect)rect {
    // Drawing code here.
}

- (void)awakeFromNib
{
    [self loadFrameSize];
    
    _date = [[NSCalendarDate calendarDate] retain];
    [self updateDisplayForDate:_date];
    
    _timer = [[NSTimer scheduledTimerWithTimeInterval:60.0
                                target:self 
                                selector:@selector(checkDateChange:) 
                                userInfo:nil 
                                repeats:YES] retain];
}

- (void)loadFrameSize
{
    NSString *lastSizeString = [[NSUserDefaults standardUserDefaults] objectForKey:@"MoonViewSize"];
    if (lastSizeString && [lastSizeString length] != 0) {
        NSSize sSize = NSSizeFromString(lastSizeString);
        [self setFrameSize:sSize];
    }
}

- (void)saveFrameSize
{
    NSString *sizeString = NSStringFromSize([self frame].size);
    [[NSUserDefaults standardUserDefaults] setObject:sizeString forKey:@"MoonViewSize"];
}

- (void)cleanup
{
    [_timer invalidate];
    [_timer release];
    _timer = nil;
    
    [_date release];
    _date = nil;
    
    [self saveFrameSize];
}


- (void)dealloc
{

    [super dealloc];
}






- (IBAction)nextMoonDate:(id)sender
{
    _isOnToday = NO;
    NSCalendarDate *newDate = [_date dateByAddingYears:0 months:0 days:1 hours:0 minutes:0 seconds:0];
    [_date release];
    _date = [newDate retain];
    [self updateDisplayForDate:_date];
}

- (IBAction)previousMoonDate:(id)sender
{
    _isOnToday = NO;
    NSCalendarDate *newDate = [_date dateByAddingYears:0 months:0 days:-1 hours:0 minutes:0 seconds:0];
    [_date release];
    _date = [newDate retain];
    [self updateDisplayForDate:_date];
}

- (IBAction)todayMoonDate:(id)sender
{
    _isOnToday = YES;
    [_date release];
    _date = [[NSCalendarDate calendarDate] retain];
    [self updateDisplayForDate:_date];
}

- (void)checkDateChange:(NSTimer *)timer
{
    /* If we are looking at the moon phase for "today", check to see if the current day is not equal to the stored day.
       Update if date has changed (IE, midnight) */

    if (!_isOnToday) return;
    
    NSCalendarDate *today   = [NSCalendarDate calendarDate];
    int nowDay              = [today dayOfYear];
    int lastDay             = [_date dayOfYear];
    
    if (nowDay != lastDay) [self todayMoonDate:nil];
}






- (void)updateDisplayForDate:(NSCalendarDate *)anyDate
{
    NSString *nDaysString, *fDaysString;
    
    int daysTillFull = [self daysUntilFullMoon:anyDate];
    NSCalendarDate *nextFullDate = [anyDate dateByAddingYears:0 months:0 days:daysTillFull hours:0 minutes:0 seconds:0];
    NSString *nextFullDateString = [nextFullDate descriptionWithCalendarFormat:@"%b %d, %Y"];
    if (daysTillFull > 1) {
        fDaysString = @"days";
    } else {
        fDaysString = @"day";
    }
    
    int daysTillNew = [self daysUntilNewMoon:anyDate];
    NSCalendarDate *nextNewDate = [anyDate dateByAddingYears:0 months:0 days:daysTillNew hours:0 minutes:0 seconds:0];
    NSString *nextNewDateString = [nextNewDate descriptionWithCalendarFormat:@"%b %d, %Y"];
    if (daysTillNew > 1) {
        nDaysString = @"days";
    } else {
        nDaysString = @"day";
    }

    [_dateField setStringValue:[NSString stringWithFormat:@"Moon Phase Information for %@", [anyDate descriptionWithCalendarFormat:@"%b %d, %Y"]]];
    [_phasePercentField setStringValue:[NSString stringWithFormat:@"Phase Percent: %0.02f%%", [self moonPhasePercent:anyDate]]];
    [_fullnessPercentField setStringValue:[NSString stringWithFormat:@"Illumination: %0.02f%%", [self moonFullness:anyDate]]];
    [_daysUntilFullField setStringValue:[NSString stringWithFormat:@"Next Full Moon: %@ (%d %@)", nextFullDateString, daysTillFull, fDaysString]];
    [_daysUntilNewField setStringValue:[NSString stringWithFormat:@"Next New Moon: %@ (%d %@)", nextNewDateString, daysTillNew, nDaysString]];
      
    int picNum = [self getMoonPhasePictureInteger:[self moonPhasePercent:anyDate]];
    [_imageView setImage:[NSImage imageNamed:[NSString stringWithFormat:@"MoonPhase%d", picNum]]]; 
}


- (int)daysUntilFullMoon:(NSCalendarDate *)anyDate
{ 
    double phasePercent;
    int daysTillFull;
    double fullness;
    
    phasePercent = [self moonPhasePercent:anyDate];
  
    if (phasePercent <= 50) {
        fullness = phasePercent * 2;
    } else {
        fullness = (100 - phasePercent) * 2;
    }
    
    daysTillFull = (1 - fullness / 100) * (double)SYNODIC;
    
    return daysTillFull;
}

- (int)daysUntilNewMoon:(NSCalendarDate *)anyDate
{
    double phasePercent;
    int daysTillNew;
  
    phasePercent = [self moonPhasePercent:anyDate];
  
    daysTillNew = (1 - phasePercent / 100) * (double)SYNODIC;
  
    return daysTillNew;
}

- (double)moonPhasePercent:(NSCalendarDate *)anyDate
{
    NSCalendarDate *baseDate;
    double aNum,bNum,diff,phase;
    double mSecPerDay;
  
    mSecPerDay = 24 * 60 * 60 * 1000;
  
    baseDate = [NSCalendarDate dateWithYear:2003 
                        month:6 day:29 hour:6 minute:53 second:0 
                        timeZone:[NSTimeZone timeZoneWithName:@"GMT"]];
  
    bNum = [baseDate timeIntervalSinceReferenceDate];
    aNum = [anyDate timeIntervalSinceReferenceDate];
  
    diff = ((aNum - bNum) * 999.7500) + mSecPerDay;
    
    if (diff < 0) diff = -diff;
  
    phase = diff / ((double)SYNODIC * mSecPerDay);
        
    phase = phase * 100;      // make it a ##.##% figure
    
    while (phase > 100) {
        phase = phase - 100;
    }
  
    return phase;
}

- (double)moonFullness:(NSCalendarDate *)anyDate
{
    double phasePercent, fullness;
  
    phasePercent = [self moonPhasePercent:anyDate];
  
    if (phasePercent <= 50) {
        fullness = phasePercent * 2;
    } else {
        fullness = ( 100 - phasePercent ) * 2;
    }
    
    return (float)[self getPrec:fullness prec:2];
}

- (double)getPrec:(double)val prec:(double)prec
{
    double v;
    v = val * pow( 10, prec );
    v = round(v);
    v = v / pow( 10, prec );
    return v;
}

- (NSString *)getMoonPhaseName:(double)phase
{
    if ( phase >= 0 && phase <= 2.49 ) {
    
        return @"New Moon";
    
    } else if ( phase >= 2.5 && phase <= 22.49 ) {
    
        return @"Waxing Crescent";
    
    } else if ( phase >= 22.5 && phase <= 27.49 ) {
    
        return @"First Quarter Moon";
    
    } else if ( phase >= 27.5 && phase <= 47.49 ) {
    
        return @"Waxing Gibbous";
    
    } else if ( phase >= 47.5 && phase <= 52.49 ) {
    
        return @"Full Moon";
    
    } else if ( phase >= 52.5 && phase <= 73.49 ) {
    
        return @"Waning Gibbous";
    
    } else if ( phase >= 73.5 && phase <= 77.49 ) {
    
        return @"Third Quarter Moon";
    
    } else if ( phase >= 77.5 && phase <= 97.49 ) {
    
        return @"Waning Crescent";
    
    } else {
  
        return @"New Moon";
    }
}

- (int)getMoonPhasePictureInteger:(double)phase
{
    if ( phase >= 0 && phase <= 2.49 ) {
    
        return 0;   //"New Moon";
    
    } else if ( phase >= 2.5 && phase <= 22.49 ) {
    
        return 1;   //@"Waxing Crescent";
    
    } else if ( phase >= 22.5 && phase <= 27.49 ) {
    
        return 2;   //@"First Quarter Moon";
    
    } else if ( phase >= 27.5 && phase <= 47.49 ) {
    
        return 3;    //@"Waxing Gibbous";
    
    } else if ( phase >= 47.5 && phase <= 52.49 ) {
    
        return 4;    //@"Full Moon";
    
    } else if ( phase >= 52.5 && phase <= 73.49 ) {
    
        return 5;    //@"Waning Gibbous";
    
    } else if ( phase >= 73.5 && phase <= 77.49 ) {
    
        return 6;   //@"Third Quarter Moon";
    
    } else if ( phase >= 77.5 && phase <= 97.49 ) {
    
        return 7;   //@"Waning Crescent";
    
    } else {
  
        return 0;   //@"New Moon";
    }
}

@end
