//
//  Node.h
//  OutlineMe
//
//  Created by John Schilling on 11/11/04.
//  Copyright 2004 John Schilling. All rights reserved.
//

#import <Cocoa/Cocoa.h>

@interface Node : NSObject {
    
    id                  _delegate;
    
    NSDictionary        *_dict;
    NSString            *_dataFolderPath;
    NSString            *_destFileName;
    NSString            *_itemName;
    NSString            *_nodeTitle;
    NSString            *_itemDescription;
    int                 _fileType;
    
    NSMutableString     *_lastUpdateString;
    NSMutableString     *_shortLastUpdateString;
    NSCalendarDate      *_lastUpdateDate;
    NSMutableString     *_nextUpdateString;
    NSMutableString     *_shortNextUpdateString;
    NSCalendarDate      *_nextUpdateDate;
    NSMutableString     *_statusMessage;
    NSMutableString     *_longStatusMessage;
    
    BOOL                _autoUpdates;
    int                 _autoUpdateRate;
    BOOL                _downloadsAtStartup;
    
    BOOL                _overrideDataMismatch;
    
    NSURLConnection     *_download;
    NSMutableData       *_receivedData;
    BOOL                _isDownloading;
    
    NSString            *_destinationFolderPath;
    NSString            *_sourceURL;
    NSString            *_filePath;
        
    int                 _bytesReceived;
    int                 _expectedContentLength;
}

- (id)initWithDictionary:(NSDictionary *)aDict 
                        delegate:(id)delegate 
                        dataFolderPath:(NSString *)dataFolderPath
                        overrideDataMismatch:(BOOL)overrideDataMismatch;
- (BOOL)isNode;
- (id)childAtIndex:(int)i;
- (BOOL)expandable;
- (void)cleanup;

- (void)setStatusMessage:(NSString *)shortMessage long:(NSString *)longMessage;
- (NSString *)itemName;
- (NSString *)itemStatus;
- (NSString *)longStatus;
- (NSString *)itemDescription;
- (NSString *)nodeTitle;
- (NSString *)dataSource;
- (NSString *)URL;
- (NSString *)lastUpdate;
- (NSString *)nextUpdate;
- (NSString *)shortLastUpdate;
- (NSString *)shortNextUpdate;
- (NSString *)filePath;
- (NSString *)fileName;
- (int)fileType;

- (BOOL)isDownloading;
- (BOOL)fileDownloaded;
- (BOOL)autoUpdates;
- (int)autoUpdateRate;
- (BOOL)downloadsAtStartup;

- (void)setAutoUpdates:(BOOL)autoUpdates;
- (void)setAutoUpdateRate:(int)rate;
- (void)setDownloadsAtStartup:(BOOL)downloadsAtStartup;

- (void)removeDataFile;
- (void)resetDownload;
- (void)beginOrCancelDownload;
- (void)beginDownloadingData;
- (void)cancelDownloading;
- (BOOL)checkAutoUpdate;

- (int)nextUpdateInMinutes;
- (void)calculateNextUpdate;
- (void)setLastUpdateNow;

@end

@interface NSObject (NodeDelegate)
- (void)nodeFileDidLoad:(Node *)node;
- (void)nodeDidFailWithError:(Node *)node error:(NSString *)error;
- (void)nodeHasMessage:(Node *)node;
- (void)nodeDidChangeUpdateSettings:(Node *)node;
@end
