//
//  SunCalc.m
//  Solscape
//
//  Created by John Schilling on 11/13/04.
//  Copyright 2004 John Schilling. All rights reserved.
//

#import "SunCalc.h"
#include <math.h>

@interface SunCalc (Private)
- (id)initWithDate:(NSDate *)date latitude:(float)latitude longitude:(float)longitude;
- (void)calculateRiseSet;
@end


@implementation SunCalc

#pragma mark
#pragma mark PRIVATE INSTANCE METHODS
#pragma mark

- (id)init
{
    if( ![super init] )
            return nil;
            
    return self;
}

- (id)initWithDate:(NSDate *)date latitude:(float)latitude longitude:(float)longitude
{
    if( ![super init] ) return nil;
        _date       = [date retain];
        _latitude   = latitude;
        _longitude  = longitude;
        _gmtOffset  = [[NSTimeZone defaultTimeZone] secondsFromGMT];
        [self calculateRiseSet];
    return self;
}

- (void)dealloc
{
    [_date release];
    [super dealloc];
}


#pragma mark
#pragma mark CONVIENENCE METHODS
#pragma mark

+ (SunCalc *)sunCalcForDate:(NSDate *)date latitude:(float)latitude longitude:(float)longitude
{
    SunCalc     *returnCalc;
    
    // Allocate new SunCalc
    returnCalc = [[[SunCalc alloc] initWithDate:date latitude:latitude longitude:longitude] autorelease];

    return returnCalc;
}




#pragma mark
#pragma mark CORE METHODS
#pragma mark

- (NSString *)sunrise
{
    return @"Unknown";
}

- (NSString *)sunset
{
    return @"Unknown";
}







- (void)calculateRiseSet
{
    float degreesToRadians, radiansToDegrees;
    int meridian;
    int solarMinutesAfterMidnight;
    NSString *solarTime;
    float declination, eotAdjustment;
    float sunRiseSetLSoTMinutes;
    int orrectedYear, correctedMonth;
    float t, G, C, L, alpha, obliquity;
    float adj;
    int dYear, dMonth, dDay, dHour, dMinute, dTotalsecs;
    int correctedYear;
    
    dYear = [[_date descriptionWithCalendarFormat:@"%Y" timeZone:nil locale:nil] intValue];
    dMonth = [[_date descriptionWithCalendarFormat:@"%m" timeZone:nil locale:nil] intValue];
    dDay = [[_date descriptionWithCalendarFormat:@"%d" timeZone:nil locale:nil] intValue];
    dHour = [[_date descriptionWithCalendarFormat:@"%H" timeZone:nil locale:nil] intValue];
    dMinute = [[_date descriptionWithCalendarFormat:@"%M" timeZone:nil locale:nil] intValue];
    dTotalsecs = [[_date descriptionWithCalendarFormat:@"%S" timeZone:nil locale:nil] intValue];
    
    //dTotalsecs = dTotalsecs + (dHour * 60 * 60);
    
    // constants
    degreesToRadians = 3.1416/180;
    radiansToDegrees = 180/3.1416;
  
    //local standard time meridian
    meridian = _gmtOffset * 15;

    // calculate solar minutes after midnight
    solarMinutesAfterMidnight = dHour * 60 + dMinute + 4 * (_longitude - meridian);
  
    // new declination and EOT calculations
    if (dMonth > 2) {
        correctedYear = dYear;
        correctedMonth = dMonth - 3;
    } else {
        correctedYear = dYear - 1;
        correctedMonth = dMonth + 9;
    }
  
    t = ((solarMinutesAfterMidnight / 60.0 / 24.0)+dDay+floor(30.6*correctedMonth+0.5)+floor(365.25*(correctedYear-1976))-8707.5)/36525.0;
  
    G = (357.528+35999.05*t) - floor((357.528+35999.05*t)/360.0)*360;
  
    C = (1.915* sin(G*degreesToRadians))+(0.020*sin(2.0*G*degreesToRadians));
  
    L = (280.460+(36000.770*t)+C)-floor((280.460+(36000.770*t)+C)/360.0)*360;
  
    alpha = L-2.466 * sin(2.0*L*degreesToRadians)+0.053 * sin(4.0*L*degreesToRadians);
  
    obliquity = 23.4393-0.013*t;
  
    //declination
    declination = atan(tan(obliquity*degreesToRadians)*sin(alpha*degreesToRadians))*radiansToDegrees;
  
    // adjustment for solar disk
    adj =  atan(50.0/60.0*degreesToRadians)*radiansToDegrees;
  
    //equation of time
    eotAdjustment = ((L-C-alpha)/15.0)*60.0;
  
    //now recalculate time more precisely
    solarMinutesAfterMidnight =  dHour * 60 + dMinute + (4*(_longitude - meridian))+eotAdjustment;
  
    if (solarMinutesAfterMidnight < 0) {
        solarMinutesAfterMidnight = solarMinutesAfterMidnight+24*60;
    }
    if (solarMinutesAfterMidnight >= 24*60) {
        solarMinutesAfterMidnight = solarMinutesAfterMidnight-24*60;
    }
  
    //clock time of sunrise and sunset
    sunRiseSetLSoTMinutes = radiansToDegrees * acos(-1.0*sin(_latitude*degreesToRadians)*sin((declination+adj)*degreesToRadians)/cos(_latitude*degreesToRadians)/cos((declination+adj)*degreesToRadians))*4;
 
    int rSec    = dTotalsecs;
    int rHour   = (12 * 60 - sunRiseSetLSoTMinutes - (4 * (_longitude - meridian)) - eotAdjustment) / 60;
    int rMin    = (12 * 60 - sunRiseSetLSoTMinutes - (4 * (_longitude - meridian)) - eotAdjustment) - rHour * 60;
    //YYYY-MM-DD HH:MM:SS
    NSCalendarDate *rise = [NSCalendarDate dateWithYear:dYear month:dMonth day:dDay 
                            hour:rHour minute:rMin second:rSec timeZone:[NSTimeZone defaultTimeZone]];
    NSString *risestr = [rise descriptionWithCalendarFormat:@"%a %m/%d/%y %I:%M %p"];

    NSLog(risestr);
    
    
    int sSec    = dTotalsecs;
    int sHour   = (12 * 60 + sunRiseSetLSoTMinutes - (4 * (_longitude - meridian))-eotAdjustment) / 60;
    int sMin    = (12 * 60 + sunRiseSetLSoTMinutes - (4 * (_longitude - meridian))-eotAdjustment)-sHour*60;
    //YYYY-MM-DD HH:MM:SS
    NSCalendarDate *set = [NSCalendarDate dateWithYear:dYear month:dMonth day:dDay 
                            hour:sHour minute:sMin second:sSec timeZone:[NSTimeZone defaultTimeZone]];
    NSString *setstr = [rise descriptionWithCalendarFormat:@"%a %m/%d/%y %I:%M %p"];

    NSLog(setstr);
    
}



@end
